
#include <stdlib.h>
#include <stdio.h>
#include "libtl_postscript.h"

int psScaleX = PS_SIZE_A4_X;

#define psX(x) (x*PS_SIZE_A4_X/psScaleX)
#define psY(y) (PS_SIZE_A4_Y-30-y*PS_SIZE_A4_X/psScaleX)

void ps_header (FILE *f, int scaleX,
                const char *strTitle, const char *strCopyright,
                double fontsize)
{
	psScaleX = (scaleX==0) ? PS_SIZE_A4_X : scaleX;
	fprintf(f,"%%!PS-Adobe-2.0 EPSF\n");
	fprintf(f,"%%%%Creator: libtl_postscript.c, Thorsten R. Lange, tl@thola.de\n");
	fprintf(f,"%%%%CreationDate:\n");
	fprintf(f,"%%%%Title: %s\n",strTitle);
	fprintf(f,"%%%%Pages: 1\n");
	fprintf(f,"%%%%EndComments\n");
	fprintf(f,"%% Copyright (c) %s\n",strCopyright);
	fprintf(f,"\n");
	fprintf(f,"%%%%Page: 1 1\n");
	fprintf(f,"newpath\n");
	fprintf(f,"\n");
	fprintf(f,"/Times-Roman findfont\n");
	fprintf(f,"%f scalefont\n",fontsize);
	fprintf(f,"setfont\n");
	fprintf(f,"\n");
}

void ps_end (FILE *f)
{
	fprintf(f,"\n");
	fprintf(f,"stroke\n");
	fprintf(f,"showpage\n");
	fprintf(f,"\n");
	fprintf(f,"%%%%EOF\n");
	fclose(f);
}

void ps_color(FILE *f, int color)
{
	int cr = (color >> 16) & 0xFF;
	int cg = (color >>  8) & 0xFF;
	int cb =  color        & 0xFF;
	fprintf(f,"%i %i %i setrgbcolor\n",cr,cg,cb);
}

void ps_text (FILE *f, int x, int y, int size, const char *s)
{
	fprintf(f,"%i %i moveto\n",psX(x),psY(y));
	if (size>0)
	{
		fprintf(f,"%i %i scale\n",size,size);
	}
	fprintf(f,"( %s ) show\nstroke\n",s);
	if (size>0)
	{
		fprintf(f,"%g %g scale\n",1.0/size,1.0/size);
	}
}

void ps_line (FILE *f, int x, int y, int x1, int y1)
{
	fprintf(f,"%i %i moveto\n",psX(x),psY(y));
	fprintf(f,"%i %i lineto\n",psX(x1),psY(y1));
	fprintf(f,"stroke\n");
}

void ps_rect_sub (FILE *f, int x, int y, int x1, int y1)
{
 	x  = psX(x );
 	y  = psY(y );
 	x1 = psX(x1);
 	y1 = psY(y1);
	fprintf(f,"%i %i moveto\n",x,y);
	fprintf(f,"%i %i lineto\n",x,y1);
	fprintf(f,"%i %i lineto\n",x1,y1);
	fprintf(f,"%i %i lineto\n",x1,y);
	fprintf(f,"%i %i lineto\n",x,y);
}

void ps_rect (FILE *f, int x, int y, int x1, int y1)
{
	ps_rect_sub( f, x, y, x1, y1 );
	fprintf(f,"stroke\n");
}

void ps_rect_fill (FILE *f, int x, int y, int x1, int y1, int color)
{
	fprintf(f,"newpath\n");
	ps_color(f,color);
	ps_rect_sub( f, x, y, x1, y1 );
	fprintf(f,"closepath\n");
	fprintf(f,"fill\n");
	fprintf(f,"stroke\n");
}

void ps_circle (FILE *f, int x, int y, int r)
{
	fprintf(f,"%i %i %i 0 360 arc\n",psX(x),psY(y),r);
	fprintf(f,"stroke\n");
}

void ps_circle_fill (FILE *f, int x, int y, int r, int color)
{
	fprintf(f,"newpath\n");
	ps_color(f,color);
	fprintf(f,"%i %i %i 0 360 arc\n",psX(x),psY(y),r);
	fprintf(f,"closepath\n");
	fprintf(f,"fill\n");
	fprintf(f,"stroke\n");
}

/* Postscript-Bitmap
	/W "width" def
	/H "height" def
	W H scale
	/pstr W string def
	W H 8
	[W 0 0 H neg 0 H]
	{currentfile pstr readhexstring pop}
	image
	"for height"
		"for width"
			"%02x"
		"\n"
*/
