
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include "libtl/libtl_string.h"
#include "libtl/libtl_printer.h"

#include "vs_forms.h"
#include "vs_ver.h"
#include "vs_sub.h"
#include "vs_language.h"
#include "vs_file.h"
#include "vs_graph.h"
#include "vs_extrem.h"

#ifndef NO_FORMS
FD_tagesliste      *fd_tagesliste;
FD_monat           *fd_monat;
FD_max_html        *fd_max_html;
FD_statistic       *fd_statistic;
#endif

char *onlyname (char *name)
{
	static char newName[20];
	int i=0;
	while (isalnum(name[i]) || name[i]==' ' || name[i]=='-' || name[i]=='#') 
		i++;
	strn0cpy(newName,name,i);
	return newName;
}

char *onlynameVSNET (char *name)
{	// TL 08.02.03
	static char newName[20];
	int i = 0;
	strcpy(newName,onlyname(name));
	while (name[i]!=' ' && name[i]!=0)
		i++;
	if (name[i]==' ')
	{
		char copyName[20];
		newName[i] = 0;
		sprintf(copyName,"%s%s",newName+i+1,newName);
		strcpy(newName,copyName);
	}
	return toupperstr(newName);
}


void AppendHTML2File( FILE* fHTML, FILE* fInput )
{
	char zeileStr[256];
	while (fgets(zeileStr,sizeof(zeileStr),fInput)!=NULL) 
	{
		replaceStrByStr(zeileStr,"%FILE%",diskLoad->name);
		replaceStrByStr(zeileStr,"%NAME%",stern.name);
		fprintf(fHTML,zeileStr);
		fputc('\n',fHTML);
	}  
}

void AppendHTML( FILE* fHTML, const char* theAppendName, const char* theAlternativeText )
{
	char filename[512];
	FILE* fTXT;
	sprintf(filename,"%s%s",pfadAblage,theAppendName);
	fTXT  = fopen(filename,"r");
	if (fTXT==NULL)	
	{
		fprintf( fHTML, "%s", theAlternativeText );
	}
 	else
	{ 
		AppendHTML2File( fHTML, fTXT );
		fclose(fTXT);
	}
}


/****************************************************************************/
/*  Einzelstern-Liste                                                       */
/****************************************************************************/
void VSNET_Einzelstern( bool standardObserverOnly )
{
	FILE   *fVSNET;
	struct RHell *hellPtr;
	double jd,tag;
	int    tag_h, tag_m;
	char   datum[128], filename[128];

	if (diskLoad==NULL) 
		sprintf(filename,"%sunknown.lst",pfadAblage);
	else
		sprintf(filename,"%s%s.lst",pfadAblage,diskLoad->name);
	fVSNET = fopen(filename,"w");
	fprintf(fVSNET,"\n");
	
	hellPtr = hellFirst;
	while (hellPtr!=NULL) 
	{
		bool ok = TRUE;
		bool isSelf = CanObserverOutput("",hellPtr) || BitTest(6,hellPtr->art);
		if (standardObserverOnly)
		{
			ok = isSelf; 
		}
		else
		{
		}
		
		if (ok) 
		{
			fprintf(fVSNET,"%.12s  ",onlynameVSNET(stern.name));
			jd = hellPtr->datum - JulDatZeitDifferenz[JulDatZone];
			JulDat2Zeit(jd,datum);
			sscanf(datum+12,"%i.%i",&tag_h,&tag_m);
			tag = (double)tag_h+(double)tag_m/60;
			datum[10] = '\0';
			fprintf(fVSNET,"%.4s%.2s%.2s",datum+6,datum+3,datum);
			sprintf(datum,"%4.3f",tag/24);
			fprintf(fVSNET,"%s",datum+1);
			fprintf(fVSNET,"  ");
			fprintf(fVSNET, Hell2Str(hellPtr->hell) );
			if (!standardObserverOnly)
			{
				if (isSelf)
					fprintf(fVSNET,"  %-3.3s",initialAAVSO);
				else
				if (hellPtr->observers)
					fprintf(fVSNET,"  %s",hellPtr->observers);
			}
			fprintf(fVSNET,"\n");
		}
		hellPtr = hellPtr->next;
	}
	
	fprintf(fVSNET,"\n\n");
	if (standardObserverOnly)
	{
		fprintf(fVSNET,"Observations by %s (%s.AAVSO)\n",bavObserver,initialAAVSO);
		fprintf(fVSNET,"Instruments: %s\n",bavInstrument);
	}
	
	fclose(fVSNET);
	strcat(filename," saved!");
#ifndef NO_FORMS
	fl_set_goodies_font(FL_BOLD_STYLE,FL_LARGE_SIZE);
	fl_show_messages(filename);
#endif
}

/****************************************************************************/
/*  Tagesliste                                                              */
/****************************************************************************/
void VSNET_Tag()
{
#ifndef NO_FORMS
	int i;
	fl_deactivate_all_forms();
	if (fd_tagesliste==NULL) 
	{
		fd_tagesliste = create_form_tagesliste();
		fl_set_input(fd_tagesliste->Tage,"1");
		fl_clear_browser(fd_tagesliste->Gruppe);
		for (i=0; i<STG__LAST; i++)
			fl_add_browser_line( fd_tagesliste->Gruppe, LS(70+i) );
	}
	if (useDateMode==USEDATE_DATUM) 
	{
		fl_set_input(fd_tagesliste->Datum,useDateStr);
	}
	else 
	{
		char d[20],s[20];
		GetDateStr(d);
		if (useDateMode==USEDATE_GESTERN)
			sprintf(s,"%.4s/%.2s/%02i",d+6,d+3,atoi(d)-1);
		else
			sprintf(s,"%.4s/%.2s/%.2s",d+6,d+3,d);
		fl_set_input(fd_tagesliste->Datum, s);
	}
	fl_set_button(fd_tagesliste->mit_negativ, tageslisteMitNegativ);
	fl_set_button(fd_tagesliste->mit_ident  , tageslisteMitIdent);
	fl_set_object_label(fd_tagesliste->LDays, LS_Days );
	fl_set_object_label(fd_tagesliste->LStart, LS(182) );
	fl_set_object_label(fd_tagesliste->LNegative, LS(183) );
	fl_set_object_label(fd_tagesliste->LIdent, LS(184) );
	fl_set_object_label(fd_tagesliste->Cancel, LS_Cancel );
	fl_set_object_label(fd_tagesliste->LObserver, LS(185) );
	fl_set_object_shortcutkey( fd_tagesliste->OK, XK_Return );
	
	fl_deselect_browser( fd_tagesliste->Gruppe );
	fl_select_browser_line( fd_tagesliste->Gruppe, 1+STG_NONE );
	fl_select_browser_line( fd_tagesliste->Gruppe, 1+STG_MIRA );
	fl_select_browser_line( fd_tagesliste->Gruppe, 1+STG_IRR );
	fl_select_browser_line( fd_tagesliste->Gruppe, 1+STG_ERUP );
	fl_select_browser_line( fd_tagesliste->Gruppe, 1+STG_NOVA );
	
	fl_show_form(fd_tagesliste->tagesliste,FL_PLACE_MOUSE,FL_FULLBORDER, LS(180) );
	fl_raise_form(fd_tagesliste->tagesliste);
#endif
}

void cb_VSNET_Tag (FL_OBJECT *ob, long mode)
{
	FILE  *fDat,*fVSNET;
	struct RDisk *loadFile;
	struct RStern star;
	struct RHell  hell;
	double tagStart,tagEnde, jd;
	int    zahlPos,zahlNeg;
	char   str[256], nameVSNET[128], observerStr[12];

	observerStr[0] = 0;
	if (scriptRun) 
	{
		GetDateStr(str);
		tagEnde  = Zeit2JulDat(str)+0.5;
		tagStart = tagEnde - ((*scriptArg=='\0') ? 1.0 : atof(scriptArg));
	} 
	else 
	{
#ifndef NO_FORMS
	  	if (mode==0) 
	  	{
	  		fl_hide_form(fd_tagesliste->tagesliste);
	  		fl_activate_all_forms();
	  		return;
	  	}
	  	CURSOR_WATCH(fd_tagesliste->tagesliste->window);
		tagStart = Zeit2JulDat((char*)fl_get_input(fd_tagesliste->Datum))+0.5;
	  	tagEnde  = tagStart+atof(fl_get_input(fd_tagesliste->Tage));
		tageslisteMitNegativ = fl_get_button(fd_tagesliste->mit_negativ);
	  	tageslisteMitIdent   = fl_get_button(fd_tagesliste->mit_ident);
	  	strn0cpy( observerStr, (char*)fl_get_input(fd_tagesliste->ObserverName), sizeof(observerStr)-1 );
#endif
	}
  
	zahlPos = 0;
	zahlNeg = 0;
	sprintf(nameVSNET,"%svsnet.lst",pfadAblage);
	fprintf(stderr,"VARSTARS: Creating %s ",nameVSNET);
	fVSNET = fopen(nameVSNET,"w");
	fprintf(fVSNET,"\n");
	loadFile = diskFirst->next;
	while (loadFile!=NULL) 
	{
		fDat = Open_STV(loadFile->name,&star);
#ifndef NO_FORMS
		if (!fl_isselected_browser_line( fd_tagesliste->Gruppe, star.gruppe+1 ))
			star.hellZahl = 0;
#endif
		while (star.hellZahl>0) 
		{
			bool isObserver = FALSE;
			Read_STV_Hell(fDat,&hell);
			isObserver = CanObserverOutput( observerStr, &hell );
			if ((tagStart<=hell.datum) && (hell.datum<=tagEnde)
			 && isObserver
			 && (tageslisteMitNegativ || (hell.hell<10000))
			 && (tageslisteMitIdent || (strchr(hell.bemerkung,'I')==NULL)) 
			) 
			{
				fprintf(fVSNET,"%-14.14s  ",onlynameVSNET(star.name));
				jd = hell.datum;
				JulDat2Zeit(jd-JulDatZeitDifferenz[JulDatZone],str);
				fprintf(fVSNET,"%.4s%.2s%.2s",str+6,str+3,str);
				sprintf(str,"%5.3f",frac(jd+0.5));
				fprintf(fVSNET,"%s  ",str+1); /* die Vorkommastelle weglassen */
				if (hell.hell<10000) 
				{
					fprintf(fVSNET," %4.1f",hell.hell*0.01);
					zahlPos++;
				} 
				else 
				{
					double mag = hell.hell-10000;
					if (mag>=1000)
						fprintf(fVSNET,"<%4.1f",mag*0.01);
					else
						fprintf(fVSNET," <%3.1f",mag*0.01);
					zahlNeg++;
				}
				if (strchr(hell.bemerkung,':')!=NULL) fprintf(fVSNET,":");
				else                                  fprintf(fVSNET," ");
				if (observerStr[0]==0) 
				{
					if (initialAAVSO[0]!='\0')
						fprintf(fVSNET,"  %s.AAVSO  ",initialAAVSO);
				} 
				else 
				{
					fprintf(fVSNET,"  %s  ",observerStr);
				}
				if (strchr(hell.bemerkung,'I')!=NULL) fprintf(fVSNET," ident."); 
				if (strchr(hell.bemerkung,'Y')!=NULL) fprintf(fVSNET," OUTBURST/ACTIVITY"); 
				fprintf(fVSNET,"\n");
				fprintf(stderr,".");
			}
			star.hellZahl--;
		}
		fclose(fDat);
		loadFile = loadFile->next;
	} /* of: bearbeite Stern */
	
	fprintf(fVSNET,"\n\n");
	if (observerStr[0]==0) 
	{
		fprintf(fVSNET,"Observations by %s (%s.AAVSO)\n",bavObserver,initialAAVSO);
		fprintf(fVSNET,"Instruments: %s\n",bavInstrument);
	} 
	else 
	{
		const RObserver* obsPtr = FindObserver(observerStr);
		if (obsPtr) 
		{
			fprintf(fVSNET,"Observations by %s (%s)\n",obsPtr->name,observerStr);
			if (obsPtr->instruments) 
				fprintf(fVSNET,"Instruments: %s\n",obsPtr->instruments);
		}
		else
		{
			fprintf(fVSNET,"Observations by %s\n",observerStr);
		}
	}
	fclose(fVSNET);
	fprintf(stderr,"\n");
	MailFile(mailVSNET,nameVSNET);

	if (!scriptRun) 
	{
#ifndef NO_FORMS
		FinishForm(fd_tagesliste->tagesliste);
#endif
	}

	fprintf(stderr,"\n          Number of observations\n");
	fprintf(stderr,"            positiv: %4i\n",zahlPos);
	fprintf(stderr,"            negativ: %4i\n",zahlNeg);
}

/****************************************************************************/
/*  Monatsliste                                                             */
/****************************************************************************/
void AAVSO_Formular_Header 
                (FILE *fLst, int einschubX, int einschubY, 
                 int seite, int zahl, char *monatStr, char *jahrStr)
{
	int z;
	for (z=1; z<einschubY; z++) 
		fputc(27,fLst); fputc(61,fLst);
	fprintf(fLst,"%.*s%i       %i\n",7+einschubX,SPACE,seite,(zahl+24)/25);
	fprintf(fLst,"\n");
	fprintf(fLst,"%.*s%s             %s\n",14+einschubX,SPACE,monatStr,jahrStr);
	fprintf(fLst,"%.*s%s\n",62+einschubX,SPACE,initialAAVSO);
	fprintf(fLst,"%.*s%s%c\n",8+einschubX,SPACE,bavObserver,(char)10);
	fprintf(fLst,"%.*s%s%c%c\n",8+einschubX,SPACE,bavStreet,(char)27,(char)61);
	fprintf(fLst,"%.*s%s%c%c\n",5+einschubX,SPACE,bavCity,(char)27,(char)61);
	fprintf(fLst,"%.*s%-15.15s      %s%c\n",7+einschubX,SPACE,bavCountry,bavZip,(char)10);
	fprintf(fLst,"%.*sUT%c%c\n",20+einschubX,SPACE,(char)27,(char)61);
	fprintf(fLst,"%.*s%s\n",10+einschubX,SPACE,bavInstrument);
	fputc(27,fLst); fputc(61,fLst); fputc(10,fLst);
	fprintf(fLst,"\n");
}

void BAV_Formular_Header (FILE *fLst, int seite, int zahl, char *monatStr, char *jahrStr)
{
	fprintf(fLst,"\n\n\n          Beobachtungen von %s (",bavObserver);
	if (strcmp(initialAAVSO,initialBAV)==0)
		fprintf(fLst,initialAAVSO);
	else
		fprintf(fLst,"%s.AAVSO / %s.BAV", initialAAVSO,initialBAV);	
	fprintf(fLst,")\n          Monat: %s %s\n",monatStr,jahrStr);
	fprintf(fLst,"          Anzahl der Beobachtungen: %i\n",zahl);
	fprintf(fLst,"          Seite %i von %i\n\n",seite, (zahl+44)/45);
}

void Monatsliste()
{
#ifndef NO_FORMS
	char d[20];

	fl_deactivate_all_forms();
	if (fd_monat==NULL) 
		fd_monat = create_form_monat();

	GetDateStr(d);
	d[5] = '\0'; 
	fl_set_input(fd_monat->Tag, "");
	fl_set_object_label(fd_monat->LDay, LS(188) );
	fl_set_input(fd_monat->Monat, d+3);
	fl_set_object_label(fd_monat->LMonth, LS(171) );
	fl_set_input(fd_monat->Jahr , d+6);
	fl_set_object_label(fd_monat->LYear, LS(172) );
	sprintf(d,"%i",monatEinschubX);
	fl_set_input(fd_monat->Spalten, d);  
	fl_set_object_label(fd_monat->LPrintCol, LS(174) );
	sprintf(d,"%i",monatEinschubY);
	fl_set_input(fd_monat->Zeilen, d);  
	fl_set_object_label(fd_monat->LPrintTop, LS(175) );
	fl_set_button(fd_monat->AAVSO_Druck,0);
	fl_set_object_label(fd_monat->AAVSO_Druck, LS(173) );
	fl_set_button(fd_monat->AAVSO,1);
	fl_set_button(fd_monat->AFOEV,0);
	fl_set_button(fd_monat->BAV,0);
	fl_set_button(fd_monat->BAV_Druck,0);
	fl_set_object_label(fd_monat->BAV_Druck, LS(176) );
	fl_set_object_label(fd_monat->OrdnungABC, LS(177) );
	fl_set_object_label(fd_monat->Cancel, LS_Cancel );
	fl_set_object_label(fd_monat->LObserver, LS(178) );
	
	fl_show_form(fd_monat->monat,FL_PLACE_MOUSE,FL_FULLBORDER, LS(170) );
	fl_raise_form(fd_monat->monat);
#endif
}

void cb_FormularMonat_TestDruck (FL_OBJECT *ob, long mode)
{
	FILE *fLst;         
	int   einschubX,einschubY,bavSeite;

#ifndef NO_FORMS
	einschubX = atoi(fl_get_input(fd_monat->Spalten));
	einschubY = atoi(fl_get_input(fd_monat->Zeilen));
#else
	einschubX = monatEinschubX;
	einschubY = monatEinschubY;
#endif

	fLst = OpenPrinter();
	AAVSO_Formular_Header(fLst,einschubX,einschubY,0,0,"00","Test");
	fputc(27,fLst); fputc(61,fLst);
	fprintf(fLst,"%.*sAnd      R       2440000.00  8.9  \n", einschubX+9, SPACE);
	for (bavSeite=2; bavSeite<=25; bavSeite++) 
		fprintf(fLst,"%c%c\n",(char)27,(char)61);
	fprintf(fLst,"\n");
	fprintf(fLst,"%.*s00000%.35s1\n",27+einschubX,SPACE,SPACE);
	fputc(12,fLst);
	ClosePrinter(fLst);
}

void cb_Monatsliste (FL_OBJECT *ob, long mode)
{
	typedef struct ROrdnung 
		{
			char             designation[15];
			struct RDisk    *ptr;
			struct ROrdnung *next;
		}           ROrdnung;
	ROrdnung     *ordFirst, *ordPtr, *ordSort;
	FILE         *fLstAAVSO,*fDat,*fAAVSOraw,*fAAVSOsum,*fAFOEV,*fBAV,*fLstBAV;
	struct RDisk *loadFile;
	struct RStern star;
	struct RHell  hell;
	double        monatStart,monatEnde;
	bool          aavsoDruck,aavsoListe,afoevListe,bavListe,bavDruck,
	              ordnungABC,neuOK;
	int           aavsoSeite,aavsoZeile,bavSeite,bavZeile;
	int           beobZahl,beobZahlKlein,sternZeile;
	int           bemerkung_pos, bemerkung_doppelpunkt;
	char          bemerkung_len_str[10];
	int           einschubX,einschubY;
	char          zeileStr[256],addStr[256];
	int           monatNr, tagNr;
	char          monatStr[4],jahrStr[6], observerStr[20];
	char          name_upper[StrLenName];
	char          nameAAVSOraw[128], nameAAVSOsum[128], nameAFOEV[128], nameBAV[128];
	const char    monatNameE[12][4] = {"JAN","FEB","MAR","APR","MAY","JUN",
	                                   "JUL","AUG","SEP","OCT","NOV","DEC"}; 
	const char    monatName[12][12] = {"Januar","Februar","Mrz","April",
	                                   "Mai","Juni","Juli","August",
	                                   "September","Oktober","November","Dezember"};

	if (scriptRun) 
	{
		ordnungABC = strchr(scriptArg,'O') || strchr(scriptArg,'o');
		aavsoDruck = strstr(scriptArg,"A+") || strstr(scriptArg,"a+");
		aavsoListe = strchr(scriptArg,'A') || strchr(scriptArg,'a');
		afoevListe = strchr(scriptArg,'F') || strchr(scriptArg,'f');
		bavListe   = strchr(scriptArg,'B') || strchr(scriptArg,'b');
		bavDruck   = strstr(scriptArg,"B+") || strstr(scriptArg,"b+");
		strn0cpy(jahrStr, scriptArg  , 4);
		strn0cpy(monatStr,scriptArg+4, 2);
		einschubX = monatEinschubX;
		einschubY = monatEinschubY;
		observerStr[0] = 0;
	}
	else
	{
#ifndef NO_FORMS
		if (mode==0) 
		{
			fl_hide_form(fd_monat->monat);
			fl_activate_all_forms();
			return;
		}
    
		ordnungABC = fl_get_button(fd_monat->OrdnungABC);
		aavsoDruck = fl_get_button(fd_monat->AAVSO_Druck);
		aavsoListe = fl_get_button(fd_monat->AAVSO);
		afoevListe = fl_get_button(fd_monat->AFOEV);
		bavListe   = fl_get_button(fd_monat->BAV);
		bavDruck   = fl_get_button(fd_monat->BAV_Druck);
		strn0cpy(jahrStr ,(char*)fl_get_input(fd_monat->Jahr),4);
		strn0cpy(monatStr,(char*)fl_get_input(fd_monat->Monat),2);
		tagNr = atoi((char*)fl_get_input(fd_monat->Tag));
		strn0cpy(observerStr,(char*)fl_get_input(fd_monat->Observer),sizeof(observerStr)-1);
		einschubX = atoi(fl_get_input(fd_monat->Spalten));
		einschubY = atoi(fl_get_input(fd_monat->Zeilen));
		CURSOR_WATCH(fd_monat->monat->window);
#endif
	}
	
	monatNr = atoi(monatStr);
	if (monatNr<0)
	{	// Spezialzeitrume
		double dummy;
		switch (monatNr) {
		case -1:	// das ganze Jahr
 		 	GetDateRangeOfMonth( "01", jahrStr, &monatStart, &dummy );
 	 		GetDateRangeOfMonth( "13", jahrStr, &dummy, &monatEnde );
			sprintf(monatStr,"all");
			break;
		case -2:	// ein AAVSO-Jahr mit Datenaufnahme (Sep-Aug)
 		 	GetDateRangeOfMonth( "01", jahrStr, &monatStart, &dummy );
			monatStart -= 31+30+31+30;	// September
 	 		GetDateRangeOfMonth( "08", jahrStr, &dummy, &monatEnde );
			sprintf(monatStr,"aavso");
			break;
		}
		
	}
	else
	{
 	 	GetDateRangeOfMonth( monatStr, jahrStr, &monatStart, &monatEnde );
		if ((monatNr>0) && (monatNr<13))
  			strcpy(monatStr,monatNameE[monatNr-1]);
		if (tagNr>0)
		{
			monatStart += tagNr-1;
			monatEnde = monatStart+1;
		}
	}

	beobZahl      = 0;
	beobZahlKlein = 0;
	
	ordFirst = NULL;
	if (!ordnungABC) 
	{ /* sortiere Sterne nach Designation */
		loadFile = diskFirst->next;
		while (loadFile!=NULL) 
		{
			fDat = Open_STV(loadFile->name,&star);
			if (aavsoDruck || aavsoListe || afoevListe || bavListe || bavDruck) 
			{
				neuOK = FALSE;
				while (star.hellZahl>0) 
				{
					bool isObserver = FALSE;
					Read_STV_Hell(fDat,&hell);
					isObserver = CanObserverOutput(observerStr,&hell);
					if ((monatStart<=hell.datum) && (hell.datum<=monatEnde) && isObserver)
					{
						beobZahl++;
						if ((hell.hell>=1380) && (hell.hell<10000))
						{	// positiv und dunkler als 13.8
							beobZahlKlein++;
						}
						if (hell.hell>=11380)
						{	// negativ und kleiner als 13.8
							beobZahlKlein++;
						}
						neuOK = TRUE;
					}
					star.hellZahl--;
				} /* of while (weitere Beobachtung vorhanden) */
			} 
			else 
				neuOK = TRUE;
			fclose(fDat);
			if (neuOK) 
			{
				ordPtr = (ROrdnung*) malloc(sizeof(ROrdnung));
				ordPtr->ptr = loadFile;
				strn0cpy(ordPtr->designation, star.designation, 15);
				if (ordFirst==NULL) 
				{
					ordFirst       = ordPtr;
					ordFirst->next = NULL;
				} 
				else if (strcmp(star.designation,ordFirst->designation)<=0) 
				{
					ordPtr->next = ordFirst;
					ordFirst     = ordPtr;
				} 
				else 
				{
					ordSort = ordFirst;
					while (ordSort!=NULL) 
					{
						if (ordSort->next==NULL) 
						{
							ordSort->next = ordPtr;
							ordPtr->next  = NULL;
							ordSort       = NULL;
						} 
						else if (strcmp(star.designation,ordSort->next->designation)<=0) 
						{
							ordPtr->next  = ordSort->next;
							ordSort->next = ordPtr;
							ordSort       = NULL;
						} 
						else 
						{
							ordSort       = ordSort->next;
						}
					}
				}
			} /* of: Stern einsortieren */
			loadFile = loadFile->next;
		} /* of: Stern untersuchen */
	} /* of: sortiere Sterne */

	fLstAAVSO = fAAVSOraw = fAAVSOsum = fAFOEV = fBAV = fLstBAV = NULL;
	// erzeuge Header von Ausgabefiles
	if (ordFirst==NULL) 
		loadFile = NULL;
	else
		loadFile = diskFirst->next;
	ordPtr   = ordFirst;
	if (aavsoDruck) 
	{
		aavsoSeite = 1;
		aavsoZeile = 0;
		fLstAAVSO = OpenPrinter();
		AAVSO_Formular_Header(fLstAAVSO,einschubX,einschubY,aavsoSeite,beobZahl,monatStr,jahrStr);
	}  
	if (aavsoListe) 
	{	// neues Format, 02.03.01
		if (observerStr[0]==0) 
		{
			//sprintf(nameAAVSOraw, "%s%.3s%s%s.raw", pfadAblage, initialAAVSO, monatStr, jahrStr+2);
			//fAAVSOraw = fopen(nameAAVSOraw,"w");
			sprintf(nameAAVSOsum, "%s%.3s%s%s.sum", pfadAblage, initialAAVSO, monatStr, jahrStr+2);
			fAAVSOsum = fopen(nameAAVSOsum,"w");
			fprintf(fAAVSOsum,"%s\n",bavObserver);
			fprintf(fAAVSOsum,"%s\n",bavStreet);
			fprintf(fAAVSOsum,"%s %s   %s\n\n",bavZip,bavCity,bavCountry);
		} 
		else 
		{
			//sprintf(nameAAVSOraw, "%s%.3s%s%s.raw", pfadAblage, observerStr, monatStr, jahrStr+2);
			//fAAVSOraw = fopen(nameAAVSOraw,"w");
			sprintf(nameAAVSOsum, "%s%.3s%s%s.sum", pfadAblage, observerStr, monatStr, jahrStr+2);
			fAAVSOsum = fopen(nameAAVSOsum,"w");
		}
		fprintf(fAAVSOsum,"Report Number:\n");
		fprintf(fAAVSOsum,"Observations for %s %s\n",monatStr,jahrStr);
		if (observerStr[0]==0)
			fprintf(fAAVSOsum,"Instruments: %s\n",bavInstrument);
		fprintf(fAAVSOsum,"Total Observations: %i\n",beobZahl);
		fprintf(fAAVSOsum,"Total Inner Sanctums: %i\n",beobZahlKlein);
		fprintf(fAAVSOsum,"AAVSO Observer Initials: %s\n", (observerStr[0]==0 ? initialAAVSO : observerStr) );
		fprintf(fAAVSOsum,"--------------------------------\n");
	}
	if (afoevListe) 
	{
		sprintf(nameAFOEV, "%safoev.lst", pfadAblage);
		fAFOEV = fopen(nameAFOEV,"w");
		if (observerStr[0]==0)
			fprintf(fAFOEV,"Observations by %s (%s)\n", bavObserver, initialAFOEV );
		else
			fprintf(fAFOEV,"Observations by %s\n", observerStr );
		fprintf(fAFOEV,"Month: %s/%02i\n\n",jahrStr,monatNr);
	}
	if (bavListe) 
	{
		sprintf( nameBAV, "%s%s%02i%s_bav.txt", pfadAblage, 
		                  (observerStr[0]==0 ? initialAAVSO : observerStr), 
		                  monatNr, jahrStr+2 );
		fBAV = fopen(nameBAV,"w");
		if (observerStr[0]==0) 
		{
			fprintf(fBAV,"Beobachtungen von %s (", bavObserver );
			if (strcmp(initialAAVSO,initialBAV)==0)
				fprintf(fBAV,"%s)", initialAAVSO);
			else
				fprintf(fBAV,"%s.AAVSO / %s.BAV)", initialAAVSO,initialBAV);	
		} else {
			fprintf(fBAV,"Beobachtungen von %s", observerStr );
		}
		fprintf(fBAV,"\nMonat: %s %s\n",monatName[monatNr-1],jahrStr);
		fprintf(fBAV,"Anzahl der Beobachtungen: %i\n\n",beobZahl);
	}
	if (bavDruck) 
	{
		bavSeite = 1;
		bavZeile = 0;
		fLstBAV = OpenPrinter();
		BAV_Formular_Header(fLstBAV,bavSeite,beobZahl,(char*)monatName[monatNr-1],jahrStr);
	}  

	// und nun jeden Stern testen und eventuell ausgeben
	while (loadFile!=NULL) 
	{
		if (!ordnungABC) loadFile = ordPtr->ptr;
		fDat = Open_STV(loadFile->name,&star);
		strcpy(name_upper,onlyname(star.name));
		toupperstr(name_upper);
		sternZeile = 0;
		while (star.hellZahl>0) 
		{
			bool isObserver = FALSE;
			Read_STV_Hell(fDat,&hell);
			isObserver = CanObserverOutput( observerStr, &hell );
			if ((monatStart<=hell.datum) && (hell.datum<=monatEnde) && isObserver)
			{
				if (aavsoDruck) {
					if (sternZeile>0)
						sprintf(zeileStr,"%.*s",17+einschubX,SPACE);
					else 
						sprintf(zeileStr,"%.*s%-8.8s %-8.8s",einschubX,SPACE,
						        star.designation,onlyname(star.name));
					sprintf(addStr,"%7.2f ",hell.datum);
					strcat(zeileStr,addStr);
					if (hell.hell<10000)
						sprintf(addStr,"%4.1f ",hell.hell*0.01);
					else 
						sprintf(addStr,"<%3.1f ",(hell.hell-10000)*0.01);
					strcat(zeileStr,addStr);
				}
				addStr[0] = '\0';
				bemerkung_pos = 0;
				bemerkung_doppelpunkt = 0;
				bemerkung_len_str[0] = '\0';
				if (hell.bemerkung[0]!=0) {
					int maxlen = 0;
					if (hell.bemerkung[0]=='*') 
						memmove(hell.bemerkung,hell.bemerkung+1,strlen(hell.bemerkung));
					while ((hell.bemerkung[bemerkung_pos]!=0) 
					       && !isdigit(hell.bemerkung[bemerkung_pos]) 
					       && hell.bemerkung[bemerkung_pos]!=' ' ) 
						bemerkung_pos++;
					if (hell.bemerkung[0]==':') 
						bemerkung_doppelpunkt = 1;
					maxlen = 4-bemerkung_pos+bemerkung_doppelpunkt;
					if (maxlen>8)	maxlen = 8;
					if (maxlen<0)	maxlen = 0;
					sprintf(bemerkung_len_str,"%.*s", maxlen, SPACE);
					strncat(addStr,hell.bemerkung,bemerkung_pos);
				} else {
					strcpy( bemerkung_len_str, "    " );
				}
				strncat(addStr,SPACE,18-strlen(addStr));
				if (aavsoDruck) {
					strcat(zeileStr,addStr);
					strcat(zeileStr," ");
					strcat(zeileStr,hell.bemerkung+bemerkung_pos);
					strncat(zeileStr,SPACE,10-strlen(hell.bemerkung)+bemerkung_pos);
					strcat(zeileStr,star.karte);
					fprintf(fLstAAVSO,"%c%c%s\n",(char)27,(char)61,zeileStr);
					aavsoZeile++;
					if (aavsoZeile==25) {
						if (aavsoSeite==1) {
							aavsoSeite = aavsoZeile;
							while (aavsoSeite<25 ) {
								fprintf(fLstAAVSO,"%c%c\n",(char)27,(char)61);
								aavsoSeite++;
							}
							aavsoSeite = 2;
							fprintf(fLstAAVSO,"\n");
							fprintf(fLstAAVSO,"%.*s%.5i%.35s%i\n",27+einschubX,SPACE,beobZahlKlein,SPACE,beobZahl);
						}
						else 
							aavsoSeite++;
						fprintf(fLstAAVSO,"%c",(char)12);
						AAVSO_Formular_Header(fLstAAVSO,einschubX,einschubY,aavsoSeite,beobZahl,monatStr,jahrStr);
						aavsoZeile   = 0;
						sternZeile =-1;
					}
				}
				if (aavsoListe) {
					if (fAAVSOraw) {
						// altes Format bis Februar 2001
						fprintf(fAAVSOraw,"%-8.8s     ", star.designation);
						fprintf(fAAVSOraw,"%-12.12s   ",name_upper);
						fprintf(fAAVSOraw,"%13.3f    ", hell.datum);
						if (hell.hell<10000)
							fprintf(fAAVSOraw,"%5.1f",(float)hell.hell*0.01);
						else 
							fprintf(fAAVSOraw,"<%4.1f",(float)(hell.hell-10000)*0.01);
						fprintf(fAAVSOraw, bemerkung_doppelpunkt ? ": " : "  ");
						fprintf(fAAVSOraw,"%.*s%s     %s\n",
										bemerkung_pos-bemerkung_doppelpunkt,
						            hell.bemerkung+bemerkung_doppelpunkt,
						            bemerkung_len_str,
		                  		(observerStr[0]==0 ? initialAAVSO : observerStr) );
						fprintf(fAAVSOsum,"%-8.8s     ",star.designation);
						fprintf(fAAVSOsum,"%-12.12s   ",name_upper);
						fprintf(fAAVSOsum,"%13.3f    ",hell.datum);
						if (hell.hell<10000)
							fprintf(fAAVSOsum,"%5.1f",(float)hell.hell*0.01);
						else 
							fprintf(fAAVSOsum,"<%4.1f",(float)(hell.hell-10000)*0.01);
						fprintf(fAAVSOsum, bemerkung_doppelpunkt ? ": " : "  ");
						fprintf(fAAVSOsum,"%.*s%s %s\n",bemerkung_pos-bemerkung_doppelpunkt,
						                                hell.bemerkung+bemerkung_doppelpunkt,
						                                bemerkung_len_str,
						                                hell.bemerkung+bemerkung_pos);
					} else {
						// neues Format, 02.03.01
						//0533+26 RR TAU    2451900.5541 13.2  Y CCDV13,14,142  SD1945  XXX
						fprintf(fAAVSOsum,"%-8.8s",star.designation);
						fprintf(fAAVSOsum,"%-10.10s",name_upper);
						fprintf(fAAVSOsum,"%11.3f ",hell.datum);
						if (hell.hell<10000)
							fprintf(fAAVSOsum,"%5.1f",(float)hell.hell*0.01);
						else 
							fprintf(fAAVSOsum,"<%4.1f",(float)(hell.hell-10000)*0.01);
						fprintf(fAAVSOsum, bemerkung_doppelpunkt ? ": " : "  ");
						fprintf(fAAVSOsum,"%.*s%s  %-11.11s%-8.8s%s\n",
										bemerkung_pos-bemerkung_doppelpunkt,
                              hell.bemerkung+bemerkung_doppelpunkt,
                              bemerkung_len_str,
                              hell.bemerkung+bemerkung_pos,
                              star.karte,
		                  		(observerStr[0]==0 ? initialAAVSO : observerStr) );
					}
				}
				if (bavListe) {
					fprintf(fBAV,"%-8.8s ", star.designation);
					fprintf(fBAV,"%-11.11s",name_upper);
					fprintf(fBAV,"%13.3f ", hell.datum);
					if (hell.hell<10000)
						fprintf(fBAV,"%5.1f",(float)hell.hell*0.01);
					else 
						fprintf(fBAV,"<%4.1f",(float)(hell.hell-10000)*0.01);
					fprintf(fBAV, bemerkung_doppelpunkt ? ": " : "  ");
					/*
					fprintf(fBAV,"%.*s%s", bemerkung_pos-bemerkung_doppelpunkt, 
					                       hell.bemerkung+bemerkung_doppelpunkt, 
					                       bemerkung_len_str);
					*/
					if (observerStr[0]==0)
						fprintf(fBAV," %s\n",initialAAVSO);
					else
						fprintf(fBAV," %s\n",observerStr);
				}
				if (bavDruck) {
					fprintf(fLstBAV,"            %-8.8s     ", star.designation);
					fprintf(fLstBAV,"%-12.12s   ",name_upper);
					fprintf(fLstBAV,"%13.3f    ", hell.datum);
					if (hell.hell<10000)
						fprintf(fLstBAV,"%5.1f ",(float)hell.hell*0.01);
					else 
						fprintf(fLstBAV,"<%4.1f ",(float)(hell.hell-10000)*0.01);
					fprintf(fLstBAV,"%.*s%s\n", bemerkung_pos, hell.bemerkung, bemerkung_len_str);
					bavZeile++;
					if (bavZeile%3==0) {
						fputc(27,fLstBAV); fputc(61,fLstBAV); 
					}
					if (bavZeile>44) {
						bavSeite++;
						fputc(12,fLstBAV);
						BAV_Formular_Header(fLstBAV,bavSeite,beobZahl,(char*)monatName[monatNr-1],jahrStr);
						bavZeile = 0;
					}
				}
				if (afoevListe) {
					fprintf(fAFOEV,"%-8.8s  ",onlyname(star.name));
					fprintf(fAFOEV,"%-5.5s%8.2f",initialAFOEV,hell.datum-2400000);
					if (strchr(hell.bemerkung,'I')!=NULL)
						fprintf(fAFOEV,":");
					else 
						fprintf(fAFOEV," ");
					if (hell.hell<10000)
						fprintf(fAFOEV," %.3iD\n",hell.hell/10);
					else 
						fprintf(fAFOEV,"<%.3iD\n",(hell.hell-10000)/10);
				}
				sternZeile++;
			}
			star.hellZahl--;
		}
		fclose(fDat);
		if (ordnungABC) {
		  loadFile = loadFile->next;
		} else {  
		  ordSort = ordPtr;
		  ordPtr  = ordPtr->next;
		  free(ordSort);
		  if (ordPtr==NULL)
		    loadFile = NULL;
		}
	} /* of: bearbeite Stern */

	if (afoevListe) 
	{
		fclose(fAFOEV);
		MailFile(mailAFOEV,nameAFOEV);
	}
	if (aavsoListe) 
	{  
		if (fAAVSOraw) 
		{
			fclose(fAAVSOraw);
			MailFile(mailAAVSO,nameAAVSOraw);
		}
		fclose(fAAVSOsum);
		MailFile(mailAAVSO,nameAAVSOsum);
	}
	if (bavListe) 
	{
		fclose(fBAV);
		MailFile(mailBAV,nameBAV);
	}
	if (aavsoDruck) 
	{
		while (aavsoZeile<25) 
		{
			fprintf(fLstAAVSO,"%c%c\n",(char)27,(char)61);
			aavsoZeile++;
		}
		fprintf(fLstAAVSO,"\n                           %.5i%.35s%i\n",beobZahlKlein,SPACE,beobZahl);
		fputc(12,fLstAAVSO);
		fl_show_message("Please insert AAVSO formulars for printing!","","");
		ClosePrinter(fLstAAVSO);
	}
	if (bavDruck) 
	{
		fputc(12,fLstBAV);
		fl_show_message("Please insert paper for printing BAV report!","","");
		ClosePrinter(fLstBAV);
	}

	if (!scriptRun) 
	{
#ifndef NO_FORMS
		FinishForm(fd_monat->monat);
#endif
	}
	
	if ((monatEinschubX!=einschubX) || (monatEinschubY!=einschubY)) 
	{
		monatEinschubX = einschubX;
		monatEinschubY = einschubY;
		monatEinschubNeu = TRUE;
	}
}


/****************************************************************************/
/*  BAV Datenbank                                                           */
/****************************************************************************/
void BAV_Datenbank_Erzeugung()
{
	struct RDisk *loadFile;
	struct RStern star;
	struct RHell  hell;
	int           bemerkung_pos, bemerkung_doppelpunkt;
	char          bemerkung_len_str[10], gcvsStar[20], gcvsLine[200];
	int           countStars=0, countObs=0;
	char          addStr[256], out_path[256], out_name[256], out_file[256];
	char          name_upper[StrLenName];
	struct RObserver *findObserverPtr;
	FILE          *fDat, *fOut;
					
	// und nun jeden Stern testen und eventuell ausgeben
	loadFile = diskFirst->next;
	while (loadFile!=NULL) 
	{
		if (strstr(loadFile->name,"_")==NULL)
		{	// Stern in Datenbank aufnehmen, die "_"-Sterne sind Riesen-Sammlungen aus der AFOEV-Datenbank
			fDat = Open_STV(loadFile->name,&star);
			strcpy(name_upper,onlyname(star.name));
			toupperstr(name_upper);
			sprintf( out_path, "%s%s", pfadBAV, loadFile->stb );
			sprintf( out_name, "%s/%s_%s", out_path, loadFile->stb, loadFile->index );
			fprintf(stderr,"Creating %s\n",out_name );
   		// Verzeichnis erzeugen
   		fOut = fopen(out_path,"r");
   		if (fOut!=NULL)
   			fclose(fOut);
   		else
   			mkdir(out_path,S_IRWXU);
   		// Info-File erzeugen
   		sprintf(out_file,"%s_Info",out_name);
   		fOut = fopen(out_file,"w");
   		fprintf(fOut,"%s\n",star.name);
   		fprintf(fOut,"%s\n",star.designation);
   		Read_GCVS_Star(Make_GCVS_Name_From_Filename(loadFile->name,gcvsStar),gcvsLine);
   		fprintf(fOut,"%06i\n", atoi(gcvsLine) );
   		fclose(fOut);
   		// Extrema-File erzeugen
   		sprintf(out_file,"%s_MinMax",out_name);
   		fOut = fopen(out_file,"w");
   		fclose(fOut);
   		// Beobachtungs-File erzeugen
   		countStars++;
   		sprintf(out_file,"%s_Obs",out_name);
   		fOut = fopen(out_file,"w");
   		while (star.hellZahl>0) 
   		{
   			bool isObserver = FALSE;
   			Read_STV_Hell(fDat,&hell);
   			isObserver = IsObservationInOrg( &hell, organisationBAV, &findObserverPtr, NULL );
   			if (isObserver)
   			{				
   				addStr[0] = '\0';
   				bemerkung_pos = 0;
   				bemerkung_doppelpunkt = 0;
   				bemerkung_len_str[0] = '\0';
   				if (hell.bemerkung[0]!=0) 
				{
   					int maxlen = 0;
   					if (hell.bemerkung[0]=='*') 
   						memmove(hell.bemerkung,hell.bemerkung+1,strlen(hell.bemerkung));
   					while ((hell.bemerkung[bemerkung_pos]!=0) 
   					       && !isdigit(hell.bemerkung[bemerkung_pos]) 
   					       && hell.bemerkung[bemerkung_pos]!=' ' ) 
   						bemerkung_pos++;
   					if (hell.bemerkung[0]==':') 
   						bemerkung_doppelpunkt = 1;
   					maxlen = 4-bemerkung_pos+bemerkung_doppelpunkt;
   					if (maxlen>8)	maxlen = 8;
   					if (maxlen<0)	maxlen = 0;
   					sprintf(bemerkung_len_str,"%.*s", maxlen, SPACE);
   					strncat(addStr,hell.bemerkung,bemerkung_pos);
   				} else {
   					strcpy( bemerkung_len_str, "    " );
   				}
   				strncat(addStr,SPACE,18-strlen(addStr));
   
     				// neues AAVSO Format, 02.03.01
     				//0533+26 RR TAU    2451900.5541 13.2  Y CCDV13,14,142  SD1945  XXX
     				//fprintf(fOut,"%-8.8s",star.designation);
     				//fprintf(fOut,"%-10.10s",name_upper);
     				fprintf(fOut,"%11.3f ",hell.datum);
     				if (hell.hell<10000)
     					fprintf(fOut,"%5.1f",(float)hell.hell*0.01);
     				else 
     					fprintf(fOut,"<%4.1f",(float)(hell.hell-10000)*0.01);
     				fprintf(fOut, bemerkung_doppelpunkt ? ": " : "  ");
     				fprintf(fOut,"%.*s%s  %-11.11s%-8.8s%s\n",
   				              bemerkung_pos-bemerkung_doppelpunkt,
                             hell.bemerkung+bemerkung_doppelpunkt,
                             bemerkung_len_str,
                             hell.bemerkung+bemerkung_pos,
                             findObserverPtr->code!=NULL ? "" : star.karte,	// Karte nur bei "Default-Beobachter"
                       		  findObserverPtr->code!=NULL ? findObserverPtr->code : initialAAVSO );
   				countObs++;
				}
				star.hellZahl--;
			}
			fclose(fOut);
			fclose(fDat);
		}
		loadFile = loadFile->next;
	} /* of: bearbeite Stern */
	
	fprintf(stderr,"Created %i stars with %i observations\n",countStars,countObs);
}


/****************************************************************************/
/*  Erzeugung_MaximaListe                                                   */
/****************************************************************************/
void Erzeugung_MaximaListe()
{
#ifndef NO_FORMS
	fl_deactivate_all_forms();
	if (fd_max_html==NULL) 
	{
		char s[20],d[20];
		double todayJD = Zeit2JulDat(GetDateStr(s));

		fd_max_html = create_form_max_html();
		JulDat2Zeit(todayJD-70,s);
		sprintf(d,"%.4s/%.2s/%.2s",s+6,s+3,s);
		fl_set_input(fd_max_html->max_von,d);
		JulDat2Zeit(todayJD+100,s);
		sprintf(d,"%.4s/%.2s/%.2s",s+6,s+3,s);
		fl_set_input(fd_max_html->max_bis,d);
		fl_set_input(fd_max_html->wochen,"4");
		fl_set_input(fd_max_html->grenzhell,"10");
		fl_set_object_label(fd_max_html->LDate1, LS(191) );
		fl_set_object_label(fd_max_html->LDate2, LS(192) );
		fl_set_object_label(fd_max_html->LCreate, LS(193) );
		fl_set_object_label(fd_max_html->LWeek, LS(194) );
		fl_set_object_label(fd_max_html->LMin, LS(195) );
		fl_set_object_label(fd_max_html->Cancel, LS_Cancel );
		fl_set_object_shortcutkey( fd_max_html->OK, XK_Return );
	}
	fl_show_form(fd_max_html->max_html,FL_PLACE_MOUSE,FL_FULLBORDER, LS(190) );
	fl_raise_form(fd_max_html->max_html);
#endif
}

int cmp_htmljump (char *jump, char *name)
{
	int  len = 0;
	char diff[4];
	int  cmp = -1;
	while (jump[len]!='\0') 
	{
		diff[len] = tolower(name[len]) - tolower(jump[len]);
		len++;
	}
	if (diff[0]<0) cmp = +1;
	else if (diff[0]==0) 
	{
		if (len>1) 
		{
	  		if (diff[1]<0) cmp = +1;
	  		else if (diff[1]==0) 
	  		{
	  			if (len>2) 
	  			{
		  			if (diff[2]<0) cmp = +1;
	  				else if (diff[2]==0) cmp = 0;
	  			} 
	  		} 
		}
	}
	return cmp;
}

void cb_Erzeugung_MaximaListe (FL_OBJECT *ob, long mode)
{
	int  	HTMLjumpCount = 15;
	char*	HTMLjumps[16] = {"A","B","C","CM","Cyg","D","G","H","L","O","P","S","T","U","V","ZZZ"};
	FILE         *fDat,*fHTML,*fTXT, *fAlert;
	struct RDisk *loadFile;
	struct RStern star;
	struct RHell  hell;
	double        monatStart,monatEnde;
	int           wochenZahl,grenzHell,aktHell,jumpNr;
	double        wocheStart,wocheEnde;
	char          zeileStr[256], str[128];
	double        todayJD, jd;
	int           mean,meanGesamt,meanMax;
	int           meanHell[10], meanZahl[10] ,meanNegativ[10];
	int           alertHell, alertPrint, alertLimit,alertFading;
	char          txtmem[1000][25];
	int           txtcount = 0;
  	
	todayJD = Zeit2JulDat(GetDateStr(zeileStr));

	if (scriptRun) 
	{
		char *c = scriptArg;
		monatStart = todayJD-70;
		monatEnde  = todayJD+100;
		wochenZahl = 4;
		grenzHell  = 1000;
		while (*c!='\0') 
		{
			if (isdigit(*c)) 
				grenzHell = 100*atoi(c);
			else if (*c=='*') 
				wochenZahl = atoi(++c);
			else if (*c=='-') 
				monatStart = todayJD-atoi(++c);
			else if (*c=='+') 
				monatEnde  = todayJD+atoi(++c);
			while (isdigit(*c)) c++;
		}
	} 
	else 
	{
#ifndef NO_FORMS
		monatStart = Zeit2JulDat( (char*)fl_get_input(fd_max_html->max_von) );
		monatEnde  = Zeit2JulDat( (char*)fl_get_input(fd_max_html->max_bis) );
		wochenZahl = atoi( fl_get_input(fd_max_html->wochen) );
		grenzHell = (int)(100*atof( fl_get_input(fd_max_html->grenzhell) ));
		if (grenzHell==0) grenzHell=9999;

		if ((mode==0) || (wochenZahl<1) || (monatEnde<monatStart)) 
		{
			fl_hide_form(fd_max_html->max_html);
			fl_activate_all_forms();
			return;
		}
		CURSOR_WATCH(fd_max_html->max_html->window);
#endif
	}
		
	jd = Zeit2JulDat("30.06.1997");
 	if (wochenZahl>9) wochenZahl=9;
	wocheEnde  = jd + 7*floor((todayJD - jd)/7);
	wocheStart = wocheEnde - 7*wochenZahl;

	fprintf(stderr,"VARSTARS: Creating vs_lp.html, vs_lp.txt and vs_alert.html\n");
	sprintf(zeileStr,"%svs_lp.html",pfadAblage);
	fHTML = fopen(zeileStr,"w");
	if (fHTML==NULL)
	{
		fprintf(stderr,"Can't open for writing: %s\n", zeileStr );
		return;
	}
	sprintf(zeileStr,"%svs_lp.txt",pfadAblage);
	fTXT  = fopen(zeileStr,"w");
	if (fTXT==NULL)
	{
		fprintf(stderr,"Can't open for writing: %s\n", zeileStr );
		fclose(fHTML);
		return;
	}
	sprintf(zeileStr,"%svs_alert.html",pfadAblage);
	fAlert  = fopen(zeileStr,"w");
	if (fAlert==NULL)
	{
		fprintf(stderr,"Can't open for writing: %s\n", zeileStr );
		fclose(fHTML);
		fclose(fTXT);
		return;
	}
 
	AppendHTML( fHTML, "vs_head.html", "<HTML>\n<HEAD>\n<TITLE>Maxima of Longperiod Stars</TITLE>\n</HEAD>\n<BODY>\n" );
	AppendHTML( fAlert, "vs_head_alert.html", "<HTML>\n<HEAD>\n<TITLE>Active Cataclysmic Stars</TITLE>\n</HEAD>\n<BODY>\n" );

	fprintf(fHTML,"<P align=center>\n");
	fprintf(fAlert,"<P align=center>\n");
	GetDateStr(zeileStr);
	GetTimeStr(str);
	fprintf(fHTML,"Update: %.10s, %.5s</P>\n",zeileStr,str);
	fprintf(fTXT ,"Update: %.10s, %.5s\n\n",zeileStr,str);
	fprintf(fAlert,"Update: %.10s, %.5s</P>\n",zeileStr,str);

	fprintf(fHTML,"<P align=center>Sternbild: \n");
	for (jumpNr=0; jumpNr<HTMLjumpCount; jumpNr++) 
	{
		fprintf(fHTML," <A HREF=\"#%s\">%s</A>",HTMLjumps[jumpNr],HTMLjumps[jumpNr]);
		if (jumpNr<HTMLjumpCount-1) 
			fprintf(fHTML,", \n");
		else 
			fprintf(fHTML,"\n");
	}
	fprintf(fHTML,"<P align=center><CENTER>\n");
	
	jumpNr = 1;
	fprintf(fHTML,"<A NAME=\"%s\">\n",HTMLjumps[0]);
	fprintf(fHTML,"<table border=5 cellpadding=2 cellspacing=4>\n");
	fprintf(fHTML,"<tr><th align=center>Stern</th><th align=center>Maximum</th>\n");

	fprintf(fAlert,"<P align=center><CENTER>\n");
	fprintf(fAlert,"<table border=5 cellpadding=2 cellspacing=4>\n");
	fprintf(fAlert,"<tr><th align=center>Stern</th><th align=center>Helligkeit</th><th align=center>Aktuell</th><th>Typ</th></tr>\n");

	for (mean=0; mean<wochenZahl; mean++) 
	{
		JulDat2Zeit(wocheStart+7*mean, zeileStr);
		fprintf(fHTML,"    <th align=center>%.4s/%.2s/%.2s</th>\n",
		              zeileStr+6, zeileStr+3, zeileStr);
	}
	fprintf(fHTML,"</tr>\n");
	
	loadFile = diskFirst->next;
	while (loadFile!=NULL) 
	{
		fDat = Open_STV(loadFile->name,&star);
		if (star.HTML) 
		{
			jd = monatStart+star.periode*fracp((star.max_jd-monatStart)/star.periode);
			if (   (jd>monatEnde)
			    && (grenzHell<HellJDindiv(todayJD,star.hellMax,star.hellMin,star.periode,
			                              star.max_jd,star.pmax,star.pab,star.pmin,star.pan)) )
				star.HTML = FALSE;
		}
		if (star.HTML || star.Alert)
		{
			alertFading  = (star.typ[0]=='R');	// e.g. RCRB, then fading alert
			// im oberen Drittel der Helligkeit fuer Ausbruchsaktivitaet, bei RCRB ein Fuenftel dunkler als normal
			alertLimit = star.hellMax + (star.hellMin-star.hellMax)/(alertFading?5:3);
			alertPrint = 0;
			alertHell  = 0;
			for (mean=0; mean<wochenZahl; mean++) 
			{
				meanHell[mean]   =0;
				meanZahl[mean]   =0;
				meanNegativ[mean]=0;
			}
			meanGesamt = 0;
			meanMax    = grenzHell;
			while (star.hellZahl>0) 
			{
				Read_STV_Hell(fDat,&hell);
				if (star.HTML && (wocheStart<=hell.datum) && (hell.datum<=wocheEnde)) 
				{
					mean = (int)((hell.datum-wocheStart)/7);
					if (hell.hell<10000) 
					{
						meanHell[mean] += hell.hell;
						meanZahl[mean]++;
						meanGesamt++;
						if (hell.hell<meanMax) 
						{
							meanMax = hell.hell;
						}
					} 
					else 
					{
						if (hell.hell>meanNegativ[mean]) 
							meanNegativ[mean] = hell.hell;
					}
				}
				if (star.Alert && todayJD-3<hell.datum)
				{	// Alarmmeldung, wenn die aktuelle Helligkeit einen Grenzwert ueberschreitet
					if ((!alertFading && hell.hell<alertLimit) || (alertFading && hell.hell>alertLimit && hell.hell<10000))
					{
						alertPrint++;
						alertHell += hell.hell;
					}
				}
				star.hellZahl--;
			}
			aktHell = HellJDindiv(todayJD,star.hellMax,star.hellMin,star.periode,star.max_jd,
			                      star.pmax,star.pab,star.pmin,star.pan);
			if (star.HTML && ((grenzHell>aktHell) || (grenzHell>meanMax))) 
			{
				fprintf(fHTML,"<tr><td align=left>");
				while (cmp_htmljump(HTMLjumps[jumpNr],loadFile->name)<=0) 
				{
					fprintf(fHTML,"<A NAME=\"%s\">",HTMLjumps[jumpNr]);
					jumpNr++;
				}
				fprintf(fHTML,"%s</td>",onlyname(star.name));
				JulDat2Zeit(jd,zeileStr);
				fprintf(fHTML,"<td align=left>%.4s/%.2s</td>",zeileStr+6,zeileStr+3);
				sprintf(txtmem[txtcount++],"%-12.12s  %.4s/%.2s",
				                           onlyname(star.name), zeileStr+6, zeileStr+3);
				for (mean=0; mean<wochenZahl; mean++) 	
				{
					fprintf(fHTML,"<td>");
					if (meanZahl[mean]>0)
						fprintf(fHTML,"%i",(int)(meanHell[mean]/meanZahl[mean]/10));
					else 
					if (meanNegativ[mean]>0)
						fprintf(fHTML,"&lt;%i",(int)((meanNegativ[mean]-10000)/10));
					else 
					if (mean==wochenZahl-1) 
					{
						star.hellZahl = wochenZahl-1;
						while (star.hellZahl>=0) 
						{
							if (meanZahl[star.hellZahl]>0) 
							{
								aktHell = aktHell+trunc(meanHell[star.hellZahl]/meanZahl[star.hellZahl])
								        - HellJDindiv( todayJD-7*(wochenZahl-star.hellZahl),
								                       star.hellMax, star.hellMin,
								                       star.periode, star.max_jd,
								                       star.pmax,star.pab,star.pmin,star.pan);
								star.hellZahl = -1;
							}
							else
							{ 
								star.hellZahl--;
							}
						}
						fprintf(fHTML,"(%i)",(int)(aktHell/10));
					}
					fprintf(fHTML,"</td>");
				}
				fprintf(fHTML,"</tr>\n");
			}
			if (alertPrint>0)
			{
				fprintf(fAlert,"<tr><td align=left>%s</td>",onlyname(star.name));
				fprintf(fAlert,"<td align=left>%4.1f - %4.1f</td>", 0.01*star.hellMax, 0.01*star.hellMin );
				fprintf(fAlert,"<td align=right>%4.1f</td>", 0.01*alertHell/alertPrint );
				fprintf(fAlert,"<td align=left>%-5.5s</td></tr>\n",star.typ);
				fprintf(stdout,"Alert: %s\t%4.1f\t%-5.5s\n",onlyname(star.name),0.01*alertHell/alertPrint,star.typ);
			}
		}
		fclose(fDat);
		loadFile = loadFile->next;
	} /* eof bearbeite Stern */
	fprintf(fHTML,"</table>\n");
	fprintf(fHTML,"</CENTER></P>\n");

	AppendHTML( fHTML, "vs_tail.html", "</BODY>\n</HTML>\n" );
	fclose(fHTML);

	txtmem[txtcount][0] = '\0';
	meanMax = (txtcount+1)/2;
	for (mean=0; mean<meanMax; mean++) 
		fprintf(fTXT,"          %s          %s\n", txtmem[mean], txtmem[meanMax+mean]);
	fputc(12,fTXT);
	fclose(fTXT);

	fprintf(fAlert,"</table>\n");
	fprintf(fAlert,"</CENTER></P>\n");
	AppendHTML( fAlert, "vs_tail_alert.html", "</BODY>\n</HTML>\n" );
	fclose(fAlert);

	if (!scriptRun) 
	{
#ifndef NO_FORMS
		FinishForm(fd_max_html->max_html);
#endif
	}
}

/****************************************************************************/
/*  Maxima_Liste                                                            */
/****************************************************************************/
void Maxima_Liste()
{
/*
TYPE PSort = ^TSort;
     TSort = RECORD
       name : string[20];
       hell : integer;
       itsMax : bool;
       periode,jd : real;
       next : PSort;
     }
VAR fDat : Text;
    loadFile : PDisk;
    l_max,l_min,l_pmax,l_pab,l_pmin,l_pan : integer;
    amplVon,amplBis,maxBis : integer;
    jdMax,l_per,l_jd,perVon,perBis,datVon,datBis : real;
    lName : string;
    lTyp,lDesignation : string[StrLenName];
    lKarte : string[StrLenKarte];
    stbVon,stbBis : string[9];
    datumSort,itsMax : bool;
    minmax : integer;
    sortFirst,sortPtr,sortVor,sortNeu : PSort;
    win : TWindow;
{
  MakeMidWindow(win,42,16,"allgemeine Maxima",FALSE);
  OpenWindow(win);
  WritePosWindow(win,1,1,"Periode von");
  WritePosWindow(win,9,2,"bis");
  WritePosWindow(win,1,3,"Amplitude von");
  WritePosWindow(win,11,4,"bis");
  WritePosWindow(win,1,5,"Maximalhelligkeit bis");
  WritePosWindow(win,1,6,"Sternbild von");
  WritePosWindow(win,11,7,"bis");
  WritePosWindow(win,1,8,"Datum von");
  WritePosWindow(win,7,9,"bis/Tage");
  WritePosWindow(win,14,1,"");
  lName:="";
  InputWindow(win,lName,15);
  if (lName!=""  perVon:=Str2Real(lname)
  else perVon:=0.000001;
  WritePosWindow(win,14,2,"");
  lName:="";
  InputWindow(win,lName,15);
  if (lName!=""  perBis:=Str2Real(lName)
  else perBis:=1e10;
  WritePosWindow(win,16,3,"");
  lname:="";
  InputWindow(win,lName,6);
  if (lName!=""  amplVon:=Trunc(Str2Real(lName)*100)
  else amplVon:=0;
  WritePosWindow(win,16,4,"");
  lname:="";
  InputWindow(win,lName,6);
  if (lName!=""  amplBis:=Trunc(Str2Real(lName)*100)+1
  else amplBis:=9999;
  WritePosWindow(win,23,5,"");
  lname:="";
  InputWindow(win,lName,6);
  if (lName!=""  maxBis:=Trunc(Str2Real(lName)*100)+1
  else amplBis:=9999;
  WritePosWindow(win,16,6,"");
  stbVon:="";
  InputWindow(win,stbVon,3);
  if (stbVon=""  stbVon:="A"
  else stbVon:=UpCaseStr(stbVon);
  WritePosWindow(win,16,7,"");
  stbBis:="";
  InputWindow(win,stbBis,3);
  if (stbBis=""  stbBis:="ZZZ"
  else stbBis:=Left(UpCaseStr(stbBis)+#255#255#255,4);
  lName:=GetDateStr;
  WritePosWindow(win,11,8,"");
  InputWindow(win,lName,15);
  datVon:=Zeit2JulDat(lName);
  lName:="";
  WritePosWindow(win,16,9,"");
  InputWindow(win,lName,15);
  if (Length(lName)>9  datBis:=Zeit2JulDat(lName)
  else {
    datBis:=Str2Real(lName);
    if (datBis<0.1  datBis:=1;
    datBis:=datVon+datBis;
    WritePosWindow(win,16,9,ZeitDatum(datBis));
  }
  WritePosWindow(win,1,11,"Sortierung nach Name (1), Datum (2) ? ");
  lName:=AskForKey("12"+_AskForReturn+_AskForEscape);
  datumSort:=(lName!="1");
  CloseWindow(win);
  if (lName=_AskForEscape  {
    EXIT;
  }
  ResetScreenColor;
  sortFirst:=NULL;
  sortVor:=NULL;
  loadFile:=diskFirst->next;
  while ((loadFile!=NULL) AND (stbVon>loadFile->name) ) loadFile:=loadFile->next;
  while ((loadFile!=NULL) AND (loadFile->name<=stbBis) ) {
    Assign(fDat,loadFile->Name+".STV");
    Reset(fDat);
    Readln(fDat,lName);
    Readln(fDat,lDesignation);
    Readln(fDat,lKarte);
    Readln(fDat,lTyp);
    Readln(fDat,l_per);
    Readln(fDat,l_jd);
    Readln(fDat,l_max);
    Readln(fDat,l_min);
    Readln(fDat,l_pmax);
    Readln(fDat,l_pab);
    Readln(fDat,l_pmin);
    Readln(fDat,l_pan);
    fclose(fDat);
    if ((perVon<=l_per) AND (l_per<=perBis) AND (l_max<=maxBis)
    AND (amplVon<=l_min-l_max) AND (l_min-l_max<=amplBis)  {
      FOR minmax:=0 TO 1 ) {
        itsMax:=(minmax=1);
        jdMax:=datVon+l_per*FracP((l_jd-datVon)/l_per);
        if ((l_pmax>50) OR (minmax=0)  {
          jdMax:=jdMax+(l_pmax*0.5+l_pab+l_pmin*0.5)*0.005*l_per;
          if (jdMax-l_per>datVon  jdMax:=jdMax-l_per;
          if (minmax=1  jdMax:=datBis;
        }
        while (jdMax<datBis ) {
          NEW(sortNeu);
          sortNeu->name:=Copy(lName+Space(20),1,20);
          sortNeu->hell:=l_max;
          sortNeu->itsMax:=itsMax;
          sortNeu->periode:=l_per;
          sortNeu->jd:=jdMax;
          if (sortFirst=NULL  {
            sortFirst:=sortNeu;
            sortNeu->next:=NULL;
            sortVor:=sortNeu;
          } else {
            if (datumSort  {
              sortPtr:=sortFirst;
              sortVor:=NULL;
              while (sortPtr!=NULL ) {
                if (sortNeu->jd<sortPtr->jd  sortPtr:=NULL
                else {
                  sortVor:=sortPtr;
                  sortPtr:=sortPtr->next;
                }
              }
              if (sortVor=NULL  {
                sortNeu->next:=sortFirst;
                sortFirst:=sortNeu;
              } else {
                sortNeu->next:=sortVor->next;
                sortVor->next:=sortNeu;
              }
            } else {
              sortVor:=sortFirst;
              while (sortVor->next!=NULL ) sortVor:=sortVor->next;
              sortVor->next:=sortNeu;
              sortNeu->next:=NULL;
            } {of Sortierungsart}
          } {of Sortierung}
          jdMax:=jdMax+l_per;
        } {of while (jdMax<datBis}
      } {of Sortiere Minimum und Maximum}
    } {of Stern im vorgegebenen Rahmen}
    loadFile:=loadFile->next;
  } {of bearbeite Stern}
  fLst = OpenPrinter();
  sortPtr:=sortFirst;
  druckTitel:="   Stern               MaxHell  Periode  Ereignis";
  Print("Periode: "+Real2Str(perVon,5,1)+" bis "+Real2Str(perBis,5,1)+" Tage");
  Print("Amplitude: "+Real2Str(amplVon*0.01,5,2)+" bis "+Real2Str(amplBis*0.01,5,2)+"mag");
  Print("");
  Print(druckTitel);
  while (sortPtr!=NULL ) {
    lName:=sortPtr->name+"  "+Real2StrLen(sortPtr->hell*0.01,4,2)+"   "
          +Real2StrLen(sortPtr->periode,4,1)+"   ";
    if (sortPtr->itsMax  lName:=lName+"MAX"
    else lName:=lName+"min";
    lName:=lName+": "+ZeitDatum(sortPtr->jd);
    Print(lName);
    sortNeu:=sortPtr;
    sortPtr:=sortPtr->next;
    free(sortNeu);
  }
  ClosePrinter;
*/
}

/****************************************************************************/
/*  SternTypTest                                                            */
/****************************************************************************/
void SternTypTest()
{
	const int maxTyp = 24;
	double hampl,hah,tp0,tp1,th0,th1,ta0,ta1;
	int typ,ts0,ts1;
	char *tn,*s0,*s1, str[512],str1[64];

	fprintf(stderr,"***\nType of %s\n", stern.name);
	if (stern.periode>0) 
	{
		hampl = (stern.hellMin-stern.hellMax)*0.01;
		if (stern.entf>0)
			hah = stern.hellMax*0.01+5-5*log10(stern.entf/3.2616);
		else 
			hah = 100.0;
		for (typ=0; typ<maxTyp; typ++) 
		{
			tp0 = -1; tp1 = -1;
			switch (typ) {
			case  1:
				tn = "Alpha CVn"; 
				tp0 = 1; tp1 = 400; th0 = 0; th1 = 0.6; ta0 = 0; ta1 = 0.2; s0 = "A0"; s1 = "F5";
				break;
			case  2:
				tn = "Beta CMa"; tp0 = 0.12; tp1 = 0.25; th0 = -5; th1 = -2.5; ta0 = 0; ta1 = 0.2; s0 = "B0"; s1 = "B3"; 
				break;
			case  3:
				tn = "Delta Cepheide"; tp0 = 1.8; tp1 = 100; th0 = -8.5; th1 = 1; ta0 = 0.1; ta1 = 2; s0 = "A0"; s1 = "G5"; 
				break;
			case  4:
				tn = "Delta Scuti"; tp0 = 0.03; tp1 = 0.25; th0 = 0; th1 = 3; ta0 = 0.1; ta1 = 1; s0 = "A2"; s1 = "F6";
				break;
			case  5:
				tn = "Mira"; tp0 = 83; tp1 = 1374; th0 = -4.5; th1 = 1; ta0 = 2; ta1 = 11; s0 = "K0"; s1 = "M9"; 
				break;
			case  6:
				tn = "RR Lyrae"; tp0 = 0.08; tp1 = 1.5; th0 = 0; th1 = 1; ta0 = 0.2; ta1 = 2; s0 = "B8"; s1 = "F2";
				break;
			case  7:
				tn = "RV Tauri"; tp0 = 30; tp1 = 150; th0 = -4; th1 = -2.5; ta0 = 0.3; ta1 = 5; s0 = "F0"; s1 = "K9";
				break;
			case  8:
				tn = "ZZ Ceti"; tp0 = 0; tp1 = 0.02; th0 = 8; th1 = 20; ta0 = 0; ta1 = 0.3; s0 = "A0"; s1 = "F9";
				break;
			case  9: 
				tn = ":BY Dra"; tp0 = 1.5; tp1 = 120; th0 = 8; th1 = 20; ta0 = 0; ta1 = 0.3; s0 = "K0"; s1 = "M9";
				break;
			case 10:
				tn = ":AM CVn"; tp0 = 0; tp1 = 0.1; th0 = 8; th1 = 20; ta0 = 100; ta1 = 100; s0 = "A0"; s1 = "F9";
				break;
			case 11:
				tn = ":AM Herculis"; tp0 = 0.04; tp1 = 0.13; th0 = 100; th1 = 100; ta0 = 100; ta1 = 100; s0 = "A0"; s1 = "M0";
				break;
			case 12:
				tn = ":W UMa"; tp0 = 0; tp1 = 0.4; th0 = 4; th1 = 6; ta0 = 0.6; ta1 = 0.9; s0 = "A0"; s1 = "K5";
				break;
			case 13:
				tn = ":UX UMa"; tp0 = 0.07; tp1 = 0.5; th0 = 100; th1 = 100; ta0 = 0.1; ta1 = 1.5; s0 = "A0"; s1 = "A9";
				break;
			case 14:
				tn = ":Beta Lyrae"; tp0 = 0.5; tp1 = 13; th0 = 100; th1 = 100; ta0 = 0.2; ta1 = 0.9; s0 = "O8"; s1 = "F1";
				break;
			case 15:
				tn = ":Algol"; tp0 = 1; tp1 = 20; th0 = 100; th1 = 100; ta0 = 0.1; ta1 = 3.5; s0 = "B0"; s1 = "A9";
				break;
			case 16:
				tn = ":Zeta Aur"; tp0 = 500; tp1 = 1000; th0 = 100; th1 = 100; ta0 = 0.2; ta1 = 0.8; s0 = "B0"; s1 = "M9";
				break;
			case 17:
				tn = "R CrB"; tp0 = 50; tp1 = 100000; th0 = -5; th1 = -4; ta0 = 2; ta1 = 10; s0 = "F0"; s1 = "K9";
				break;
			case 18:
				tn = "U Gem"; tp0 = 20; tp1 = 600; th0 = 7.5; th1 = 10; ta0 = 1.5; ta1 = 7; s0 = "G0"; s1 = "G9";
				break;
			case 19:
				tn = "Z Cam"; tp0 = 13; tp1 = 25; th0 = 7.5; th1 = 10; ta0 = 3; ta1 = 5; s0 = "G0"; s1 = "G9";
				break;
			case 20:
				tn = "SU UMa"; tp0 = 13; tp1 = 34; th0 = 100; th1 = 100; ta0 = 3; ta1 = 6; s0 = "  "; s1 = "  ";
				break;
			case 21:
				tn = "SRA"; tp0 = 20; tp1 = 2000; th0 = -10; th1 = 0; ta0 = 0; ta1 = 1; s0 = "M0"; s1 = "M9";
				break;
			case 22:
				tn = "SRB"; tp0 = 20; tp1 = 2000; th0 = -10; th1 = 0; ta0 = 0; ta1 = 2; s0 = "K0"; s1 = "M9";
				break;
			case 23:
				tn = "SRC-My Cephei"; tp0 = 500; tp1 = 10000; th0 = -10; th1 = -1; ta0 = 0.3; ta1 = 1.8; s0 = "G8"; s1 = "M9";
				break;
			case 24:
				tn = "SRD"; tp0 = 100; tp1 = 2000; th0 = -10; th1 = -2; ta0 = 0; ta1 = 2; s0 = "F0"; s1 = "K9";
				break;
			}
			ts0 = GetSpektrumNr(s0);
			ts1 = GetSpektrumNr(s1);
			if ((tp0<=stern.periode) && (stern.periode<=tp1) )
				if (th0==100.0 || hah==100.0 || ((th0<=hah) && (hah<=th1)) ) 
					if (ta0==100.0 || ((ta0<=hampl) && (hampl<=ta1)) ) 
						if (ts0*stern.spektralklasse==0 || ((ts0<=stern.spektralklasse) && (stern.spektralklasse<=ts1)) )  
						{
							fprintf(stderr, "%s\t", tn );
							if ((stern.entf==0) && (th0<100.0)) 
							{
								double dist = 3.2616* pow(10,(stern.hellMax*0.01+5-th0-(th1-th0)*0.5)*0.2 );
								fprintf(stderr," possible distance= %10.0f ly", dist);
							}
							fprintf(stderr,"\n");
						}
		} /* eof for */
		GetTimeStr(str1);
		GetDateStr(str);
		strcat(str,",");
		strcat(str,str1);
		ta0 = Zeit2JulDat(str);
		tp0 = ta0+stern.periode*fracp((stern.max_jd-ta0)/stern.periode);
		fprintf(stderr," next max at %s\n",ZeitDatum(tp0,str));
		tp0 = tp0+(stern.pmax*0.5+stern.pab+stern.pmin*0.5)*0.005*stern.periode;
		if (tp0-stern.periode>ta0)
			tp0 = tp0-stern.periode;
		fprintf(stderr," next min at %s\n",ZeitDatum(tp0,str));
	}
	if (stern.periode<0) 
	{
		if (stern.periode<-500) 		tn = "c";
		else if (stern.periode<-100) 	tn = "b";
		else 									tn = "a";
		fprintf(stderr,"Nova- N%s",tn);
		if (stern.entf==0) {
			double dist = pow(10,((stern.hellMax*0.01-(-11.5+2.5*log10(-stern.periode))+5.0)*0.2)+0.5) *0.001;
			fprintf(stderr," possible distance= %10.0f kpc", dist );
		}
		fprintf(stderr,"\n");
	}
}


/****************************************************************************/
/*  AllStarsStatistics                                                      */
/****************************************************************************/

#ifndef NO_FORMS

void AllStarsStatistics()
{
	int i;
	char d[20];
	fl_deactivate_all_forms();
	
	if (fd_statistic==NULL) 
	{
		struct ROrganisation *orgPtr = organisationFirst;
		fd_statistic = create_form_statistic();
		fl_set_button( fd_statistic->UseDefaultObserver, TRUE );
		fl_set_object_label( fd_statistic->UseDefaultObserver, LS(179) );
		fl_set_button( fd_statistic->UseObserver, FALSE );
		fl_set_object_label( fd_statistic->LObserver, LS(178) );
		fl_set_object_label( fd_statistic->LGroup, LS(164) );
		fl_clear_choice( fd_statistic->SelectGroup );
		fl_addto_choice( fd_statistic->SelectGroup, LS(163) );
		while (orgPtr) 
		{
			fl_addto_choice( fd_statistic->SelectGroup, orgPtr->code );
			orgPtr = orgPtr->next;
		}
		fl_set_choice( fd_statistic->SelectGroup, 5 );	// BAV
		fl_set_button( fd_statistic->UseGroup, TRUE );
		fl_clear_browser(fd_statistic->StarClass);
		for (i=0; i<STG__LAST; i++)
			fl_add_browser_line( fd_statistic->StarClass, LS(70+i) );
		fl_deselect_browser( fd_statistic->StarClass );
		//fl_select_browser_line( fd_statistic->StarClass, 1+STG_NONE );
		//fl_select_browser_line( fd_statistic->StarClass, 1+STG_MIRA );
		//fl_select_browser_line( fd_statistic->StarClass, 1+STG_IRR );
		fl_select_browser_line( fd_statistic->StarClass, 1+STG_ERUP );
		fl_select_browser_line( fd_statistic->StarClass, 1+STG_NOVA );
		fl_set_button( fd_statistic->UseStarClass, TRUE );
		fl_set_object_label( fd_statistic->UseLoadedStar, LS(270) );
		
		// Monat/Datum
		fl_set_button( fd_statistic->UseMonth, FALSE );
		fl_set_object_label( fd_statistic->UseMonth, LS(171) );
		GetDateStr(d);
		d[5] = '\0'; 
		fl_set_input(fd_statistic->SelectMonth, d+3);
		fl_set_input(fd_statistic->SelectYear , d+6);
	}
		
	fl_show_form(fd_statistic->statistic,FL_PLACE_MOUSE,FL_FULLBORDER, LS(250) );
	fl_raise_form(fd_statistic->statistic);
}

void cb_Statistic (FL_OBJECT *ob, long mode)
{
	struct ROrganisation *stat_groupPtr = NULL;
	char observerStr[128];
	int stat_group;
	bool useStandardObserver, useObserver, useGroup, useStarClass, useLoadedStar, useMonth, formatLaTeX;
	struct RDisk *loadFile;
	char pathOut[256], str[32], date0[32], date1[32];
	FILE *fOut;
	int totalCount = 0;
	int numberOfObservers = 0;		// 25.02.01
	int nUnknownObserver = -1;
	int *countObsActivity = NULL;
	double useMonthFrom=0, useMonthTo=0;	// 09.03.03
			
  	if (mode==0) 
  	{	// Cancel
  		fl_hide_form(fd_statistic->statistic);
  		fl_activate_all_forms();
  		return;
  	}
  	CURSOR_WATCH(fd_statistic->statistic->window);

	stat_group = fl_get_choice( fd_statistic->SelectGroup );
	if (stat_group<2) 
	{
		stat_groupPtr = NULL;
	} 
	else 
	{
		stat_groupPtr = organisationFirst;
		stat_group--;
		while (--stat_group>0) 
		{
			stat_groupPtr = stat_groupPtr->next;
		}
	}
	strcpy( observerStr, fl_get_input(fd_statistic->SelectObserver) );
	useStandardObserver = fl_get_button( fd_statistic->UseDefaultObserver );
	useObserver = fl_get_button( fd_statistic->UseObserver );
	useGroup = fl_get_button( fd_statistic->UseGroup );
	useStarClass = fl_get_button( fd_statistic->UseStarClass );
	useLoadedStar = fl_get_button( fd_statistic->UseLoadedStar );
	useMonth = fl_get_button( fd_statistic->UseMonth );
	if (useMonth)
		GetDateRangeOfMonth( fl_get_input(fd_statistic->SelectMonth),
		                     fl_get_input(fd_statistic->SelectYear),
		                     &useMonthFrom, &useMonthTo );
	formatLaTeX = fl_get_button( fd_statistic->LaTeXFormat );

	if (useObserver || useGroup) 
	{
		RObserver *obsPtr = observerFirst;
		while (obsPtr) 
		{
			numberOfObservers++;
			obsPtr = obsPtr->next;
		}
		nUnknownObserver = numberOfObservers++;
	}
	if (useStandardObserver)
		numberOfObservers++;
	countObsActivity = (int*) malloc( sizeof(int) * numberOfObservers );
	if (countObsActivity)
		memset( countObsActivity, 0, sizeof(int) * numberOfObservers );

	sprintf(pathOut,"%sstatistic.txt",pfadAblage);
	fOut = fopen(pathOut,"w");
	if (fOut==NULL)
		fOut = stderr;
	if (formatLaTeX)
		fprintf( fOut, "Stern & Anzahl \\\\\n" );
	else
		fprintf( fOut, "Stern                      erste Beob.  letzte B.   Helligkeit   Anzahl\n" );
	
	if (useLoadedStar)	loadFile = diskLoad;
	else						loadFile = diskFirst->next;
	while (loadFile!=NULL) 
	{
		double obsJDfirst = -1, obsJDlast = -1;
		int obsCount = 0;
		struct RStern star;
		struct RHell  hell;
		FILE* fDat = Open_STV(loadFile->name,&star);
		if (useStarClass && !fl_isselected_browser_line( fd_statistic->StarClass, star.gruppe+1 ))
			star.hellZahl = 0;
		while (star.hellZahl>0) 
		{
			bool doCountStandard = FALSE, doCountObserver = FALSE, inDate = FALSE;
			int seenByObsN = -1;
			Read_STV_Hell(fDat,&hell);
			if (useStandardObserver && (hell.art & 31)!=0)
				doCountStandard = TRUE;
			if (useObserver && observerStr[0]!=0) 
			{
				if (CanObserverOutput(observerStr,&hell))
					doCountObserver = TRUE;
			}
         if (useGroup && (stat_groupPtr==NULL || IsObservationInOrg(&hell,stat_groupPtr,NULL,&seenByObsN)) )
         	doCountObserver = TRUE;
         if (useMonth)
         	inDate = (useMonthFrom<=hell.datum && hell.datum<=useMonthTo);
         else
         	inDate = (besDatumVon<=hell.datum && hell.datum<=besDatumBis);
			if ((doCountStandard || doCountObserver) && inDate)
			{
				if (obsCount==0)
					obsJDfirst = hell.datum;
				obsJDlast = hell.datum;
				obsCount++;
				totalCount++;
				sprintf(str,"%i",totalCount);
				fl_set_object_label( fd_statistic->LCount, str );
				if ((totalCount%50)==0)
					fl_redraw_object(fd_statistic->LCount);
					//fl_redraw_form(fd_statistic->statistic);
				if (countObsActivity) 
				{
					if (doCountStandard)
						countObsActivity[numberOfObservers-1]++;
					else
					if (doCountObserver) 
					{
						int n = (seenByObsN>=0 ? seenByObsN : nUnknownObserver);
						countObsActivity[n]++;
					}
				}
			}
			star.hellZahl--;
			if (hell.observers) 
			{	// like DeleteHell()
				free(hell.observers);
				hell.observers = NULL;
			}
		}
		if (obsCount>0) 
		{
			if (formatLaTeX)
				fprintf(fOut, "%-12s & %6i \\\\\n",
			   	           onlyname(star.name), obsCount );
			else
				fprintf(fOut, "%-12s %-12s  %10.10s  %10.10s   %4.1f-%04.1f   %6i\n", 
			   	           star.designation, onlyname(star.name),
				              ZeitDatum(obsJDfirst,date0), ZeitDatum(obsJDlast,date1),
				              (float)star.hellMax/100, (float)star.hellMin/100,
				              obsCount );
		}
		fclose(fDat);
		if (useLoadedStar)	loadFile = NULL;
		else						loadFile = loadFile->next;
		//if (totalCount>10000)	loadFile = NULL;	// zum Testen
	}

	fprintf(fOut, "\nGesamt:                                                         %7i\n", totalCount );

	if (countObsActivity) 
	{
		if (formatLaTeX)
			fprintf(fOut,"\nObserver & Code & Number \\\\\n");
		else
			fprintf(fOut,"\nNumber   Observer\n");
		if (useObserver || useGroup) 
		{
			int i = 0;
			RObserver *obsPtr = observerFirst;
			while (obsPtr)
			{
				if (countObsActivity[i])
				{
					if (formatLaTeX)
						fprintf(fOut,"%-30s & %4.4s & %6i \\\\\n", obsPtr->name, obsPtr->code, countObsActivity[i] );
					else
						fprintf(fOut,"%6i   %4.4s %s\n", countObsActivity[i], obsPtr->code, obsPtr->name );
				}
				i++;
				obsPtr = obsPtr->next;
			}
			if (countObsActivity[nUnknownObserver]>0)
			{
				if (formatLaTeX)
					fprintf(fOut,"%-30s & %4.4s & %6i \\\\\n", "(unknown)", "???", countObsActivity[nUnknownObserver] );
				else
					fprintf(fOut,"%6i   ? (unknown)\n", countObsActivity[nUnknownObserver] );
			}
		}
		if (useStandardObserver)
		{
			if (initialAAVSO[0])
				fprintf(fOut,"%6i   %4.4s %s\n", countObsActivity[numberOfObservers-1], initialAAVSO, bavObserver );
			else
				fprintf(fOut,"%6i   Standard-Observer\n", countObsActivity[numberOfObservers-1] );
		}
		free(countObsActivity);
	}
		
	if (fOut!=stderr)
		fclose(fOut);

	if (!scriptRun) 
	{
		FinishForm(fd_statistic->statistic);
	}
}

#endif

/****************************************************************************/
/*  Erzeugung_Stern_HTML                                                    */
/****************************************************************************/
void Erzeugung_Stern_HTML( bool toStdout, const char* theImgPath )
{
	const char *FORM_NAME[] = { "schwarzer Kreis", "roter Kreis", "gelber Kreis", "blauer Kreis", "grner Kreis",
	                            "schwarzes Quadrat", "rotes Quadrat", "gelbes Quadrat", "blaues Quadrat", "grnes Quadrat" };
	int	i, outGraphIndex = 0;
	FILE	*fHTML, *fHTMLmini;
	char	zeileStr[512], graphFilename[512];
	double	jdToday;
	float	grPerioden = 3.0;
	float	grDateStart = 0, grDateEnd = 0;
	bool	grAlle = false, grBAV = false;
	bool	grListe = false;
	int	groupForm = 0;
	char	grObsName[HELL_OBS_N][32];
	int	grObsForm[HELL_OBS_N] = {0,1,2};
	bool	grBRDiagramm = false;

	for (i=0; i<HELL_OBS_N; i++)
		grObsName[i][0] = 0;

	if (scriptRun) 
	{
		char *ptr;
		mitTheoretKurve = FALSE;
		ptr = strstr(scriptArg,"bmr=");
		if (ptr)
			grBRDiagramm = true;
		ptr = strstr(scriptArg,"listobs=");
		if (ptr)
			grListe = TRUE;
		ptr = strstr(scriptArg,"per=");
		if (ptr)
			grPerioden = atof(ptr+4);
		ptr = strstr(scriptArg,"date0=");
		if (ptr)
		{
			grDateStart = atof(ptr+6);
			if (grDateStart < 1)
				;	// not defined
			else
			if (grDateStart < 40)
				grDateStart = Zeit2JulDat(ptr+6);
			else
			if (grDateStart < 2450000)
				grDateStart += 2450000;
		}
		ptr = strstr(scriptArg,"date1=");
		if (ptr)
		{
			grDateEnd = atof(ptr+6);
			if (grDateEnd < 1)
				;	// not defined
			else
			if (grDateEnd < 40)
				grDateEnd = Zeit2JulDat(ptr+6);
			else
			if (grDateEnd < 2450000)
				grDateEnd += 2450000;
		}
		ptr = strstr(scriptArg,"group=");
		if (ptr)
		{	int mode = atoi(ptr+6);
			grAlle = ((mode&1)>0);
			grBAV  = ((mode&2)>0);
		}
		ptr = strstr(scriptArg,"groupform=");
		if (ptr)
			groupForm = atoi(ptr+10);
		for (i=0; i<HELL_OBS_N; i++)
		{	char name[16];
			sprintf(name,"obs%i=",i+1);
			ptr = strstr(scriptArg,name);
			if (ptr)
			{	strncpy(grObsName[i],ptr+5,30);
				grObsName[i][30] = 0;
				ptr = strchr(grObsName[i],',');
				if (ptr)
					*ptr = 0;
				toupperstr(grObsName[i]);
			}
			sprintf(name,"obs%iform=",i+1);
			ptr = strstr(scriptArg,name);
			if (ptr)
				grObsForm[i] = atoi(ptr+strlen(name));
		}
	}
	else
	{
	}

	sprintf(zeileStr,"%s%s.html",pfadAblage,diskLoad->name);
	if (toStdout)
		fHTML = stdout;
	else
		fHTML = fopen(zeileStr,"w");
	if (fHTML==NULL)
	{
		fprintf(stderr,"VARSTARS: Can't open %s for writing!\n",zeileStr);
		return;
	}
	sprintf(zeileStr,"%s%s-print.html",pfadAblage,diskLoad->name);
	fHTMLmini = fopen(zeileStr,"w");
	if (!toStdout)
		fprintf(stderr,"VARSTARS: Creating %s.html\n",diskLoad->name);

	sprintf( zeileStr, "<HTML>\n<HEAD>\n<TITLE>Star information: %s</TITLE>\n</HEAD>\n<BODY>\n", diskLoad->name );
	AppendHTML( fHTML, "star_head.html", zeileStr );
	if (fHTMLmini)
		fprintf( fHTMLmini, "<HTML>\n<HEAD>\n<TITLE>BAV Lichtkurve: %s</TITLE>\n</HEAD>\n<BODY>\n<B>BAV %s: %s</B><P>\n", 
		         diskLoad->name, grBRDiagramm?"B-R Diagramm":"Lichtkurve", stern.name );

	fprintf(fHTML,"<H3>Sterndaten</H3>\n");
	fprintf(fHTML,"<P>\n");
	fprintf(fHTML,"<UL>\n");
	fprintf(fHTML,"<LI> Helligkeit: %5.2f bis %5.2f mag\n", (float)stern.hellMax/100, (float)stern.hellMin/100 );
	fprintf(fHTML,"<LI> Periode: %g Tage\n", stern.periode );
	fprintf(fHTML,"<LI> Designation: %s\n",stern.designation);
	fprintf(fHTML,"<LI> Typ: %s\n",stern.typ);
	fprintf(fHTML,"</UL>\n");
	fprintf(fHTML,"<P>\n");
	fprintf(fHTML,"<H3>%s</H3>\n", grBRDiagramm?"B-R Diagramm":"Lichtkurve" );
	fprintf(fHTML,"Created by %s %s (%s@%s), version %s\n", VerError, VerAutor, VerEMail1, VerEMail2, VerDatum );
	fprintf(fHTML,"<P>\n");

	if (!toStdout)
		fprintf(stderr,"VARSTARS: Creating %s.gif\n",diskLoad->name);
	HellKurve_Reset();
	ExtremGraph_Reset();

	jdToday = Zeit2JulDat(GetDateStr(zeileStr)) +0.5;
	if (grPerioden>0)
	{
		double jd0 = jdToday - grPerioden * fabs(stern.periode);
		HellKurve_Datumsbereich(jd0,jdToday);
		ExtremGraph_DateRange(jd0,jdToday);
	}
	if (grDateStart > 0)
	{
		double jd1 = (grDateEnd > 0 ? grDateEnd : jdToday);
		HellKurve_Datumsbereich(grDateStart,jd1);
		ExtremGraph_DateRange(grDateStart,jd1);
	}
	else
	if (grDateEnd > 0 && grPerioden > 0)
	{
		double jd0 = grDateEnd - grPerioden * fabs(stern.periode);
		HellKurve_Datumsbereich(jd0,grDateEnd);
		ExtremGraph_DateRange(jd0,grDateEnd);
	}

	HellKurve_Gruppe( -1, grAlle?organisationBAV:NULL, 0 );
	if (grBAV)
	{
		HellKurve_Gruppe( 1, organisationBAV, groupForm );
	}

	// Standard-Beobachter erst mal ausschalten
	HellKurve_Beobachter( -1, NULL, -1 );
	for (i=0; i<HELL_OBS_N; i++)
		if (grObsName[i][0])
		{
			if (initialAAVSO[0] && strcmp(grObsName[i],initialAAVSO)==0)
				HellKurve_Beobachter( -1, initialAAVSO, grObsForm[i] );
			else
				HellKurve_Beobachter( i, grObsName[i], grObsForm[i] );
		}

	{	// Zeitindex fr Browser-Reload
		struct timeb tb;
		ftime(&tb);
		outGraphIndex = tb.millitm;
	}

	// Grafik erzeugen
	sprintf(graphFilename,"%s-%i", diskLoad->name, outGraphIndex );
	if (grBRDiagramm)
	{
		ExtremGraphPaint(2,graphFilename);
	}
	else
	{
		HellKurvePaint(2,graphFilename);
		if (grListe)
			HellKurvePaint(4,graphFilename);
	}

	fprintf(fHTML,"<IMG src=\"%s%s.gif\" alt=\"%s von %s\"><BR>\n", theImgPath, graphFilename, (grBRDiagramm?"B-R-Diagramm":"Lichtkurve"), stern.name );
	if (fHTMLmini)
		fprintf(fHTMLmini,"<IMG src=\"%s%s.gif\" alt=\"%s von %s\"><BR>\n", theImgPath, graphFilename, (grBRDiagramm?"B-R-Diagramm":"Lichtkurve"), stern.name );

	for (i=0; i<HELL_OBS_N; i++)
		if (grObsName[i][0])
		{
			fprintf(fHTML,"%s (%s),\n", grObsName[i], FORM_NAME[grObsForm[i]] );
			if (fHTMLmini)
				fprintf(fHTMLmini,"%s (%s),\n", grObsName[i], FORM_NAME[grObsForm[i]] );
		}

	if (grBAV)
	{
		fprintf(fHTML,"BAV (%s),\n", FORM_NAME[groupForm] );
		if (fHTMLmini)
			fprintf(fHTMLmini,"BAV (%s),\n", FORM_NAME[groupForm] );
	}
	if (grAlle)
	{
		fprintf(fHTML,"VSNET (%s),\n", FORM_NAME[0] );
		if (fHTMLmini)
			fprintf(fHTMLmini,"VSNET (%s),\n", FORM_NAME[0] );
	}
	if (fHTMLmini)
	{
		fprintf(fHTML,"<P><A HREF=\"%s%s-print.html\">Druck-Version</A> dieser Seite</P>\n", theImgPath, diskLoad->name );
	}
	if (grListe)
	{
		fprintf(fHTML,"<P><A HREF=\"%s%s.lst\">Liste</A> der Einzelbeobachtungen</P>\n", theImgPath, graphFilename );
	}

	AppendHTML( fHTML, "star_tail.html", "</BODY>\n</HTML>\n" );
	fclose(fHTML);
	if (fHTMLmini)
	{
		fprintf( fHTMLmini, "</BODY>\n</HTML>\n" );
		fclose(fHTMLmini);
	}
}



