/* EXTREMA: Minima und Maxima, B-R-Diagramm
   TL 15.-16.05.04: erste Version
   TL 25.07.04: ExtremConvertBAV short-period
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include "vs_forms.h"
#include "vs_file.h"
#include "libtl/libtl_string.h"
#include "libtl/libtl_printer.h"
#include "libtl/libtl_draw.h"

#include "vs_sub.h"
#include "vs_forms.h"
#include "vs_language.h"
//#include "vs_graph.h"
#include "vs_extrem.h"

#ifndef NO_FORMS
extern FD_graf_phase 	*fd_graf_phase;
#else
FD_FORM  		*fd_graf_phase;
#endif


char* ExtremGetFileName( const char* theStarName, char* theFileName )
{
	sprintf(theFileName,"%s%s.ste",pfadSTV,theStarName);
	return theFileName;
}


bool ExtremExists( const char* theName )
{
	FILE*   fDat;
	char    filename[128];
	fDat = fopen(ExtremGetFileName(theName,filename),"r");
	if (fDat==NULL)
		return false;
	fclose(fDat);
	return true;
}


bool ExtremLoad( const char* theName )
{
	FILE*   fDat;
	char    filename[128], line[512], obsCode[StrLenObsCode];
	int	extType, extMag;
	double	extDate;
	int	n = 0, pos;

	strcpy(activeSTEFilename,theName);
	fDat = fopen(ExtremGetFileName(theName,filename),"r");
	if (fDat==NULL)
	{
		fprintf(stderr,"Open_STE: File not found: '%s'\n",filename);
		return false;
        }

	while (fgetline(line,sizeof(line),fDat)!=NULL)
	{
		int rn = sscanf(line,"%i %lf %i %s %n", &extType, &extDate, &extMag, obsCode, &pos );
		n++;
		if (rn==4)
		{
			ExtremNew( extType, extDate, extMag, obsCode, line+pos );
		}
		else
		{
			fprintf(stderr,"Open_STE: Incomplete line %i\n",n);
		}
	}

	fclose(fDat);
	stern.extremZahl = n;
	return true;
}


struct RExtrem* ExtremNew( int theTyp, double theDate, int theMag, const char* theObsCode, const char* theComment )
{
	struct RExtrem *extremNew = (struct RExtrem*) malloc( sizeof(struct RExtrem) );
	extremNew->typ   = theTyp;
	extremNew->datum = theDate;
	extremNew->hell  = theMag;
	if (theObsCode)
		strncpy( extremNew->observer, theObsCode, StrLenObsCode );
	else
		extremNew->observer[0] = '\0';
	extremNew->observer[StrLenObsCode-1] = '\0';
	if (theComment)
		strncpy( extremNew->bemerkung, theComment, StrLenBemerkung );
	else
		extremNew->bemerkung[0] = '\0';
	extremNew->bemerkung[StrLenBemerkung-1] = '\0';
	extremNew->next = extremFirst;
	extremFirst = extremNew;
	return extremNew;
}


void ExtremDelete()
{
	struct RExtrem *ptr;
	while (extremFirst)
	{
		ptr = extremFirst;
		extremFirst = extremFirst->next;
		free(ptr);
	}
	stern.extremZahl = 0;
}


#define GRAF_HELL_LEFT  	40
#define GRAF_HELL_RIGHT 	10
#define GRAF_HELL_TOP		30
#define GRAF_HELL_BOTTOM	30

double extrem_jd0, extrem_jd1;
int    extrem_typ = 1;	// Maximum
int    extremEpocheStart, extremEpocheEnd;
double extremEpocheStep;

void ExtremGraphPaint( int theMode, const char* theFileName )
{
	int           wx,wy,wb,wh;
	struct RExtrem *extremPtr;
	PDrawBitmap   db;
	int stepPhase = 10;
	int           i, x,y, epocheTotalN;
	double        hdif, epocheNullJD, epocheNullX;
	int           epocheGridStep;
	char          str[512];

	/* Ausmae der Grafik */
	wx = GRAF_HELL_LEFT;
	wy = GRAF_HELL_TOP;
	draw_setup_format( grafikDruckFormat );
	draw_setup_size( grafikDruckX, grafikDruckY );
	db = draw_init( theMode ? 0 : fd_graf_phase->grafik ,&wb,&wh, pfadAblage, 
	                theFileName ? theFileName : diskLoad->name );
	if (db==NULL) return;
	
	wb = wb-wx-GRAF_HELL_RIGHT;
	wh = wh-wy-GRAF_HELL_BOTTOM;
	draw_line(db,wx,wy,wx,wy+wh,FL_WHITE);
	draw_line(db,wx,wy+wh,wx+wb,wy+wh,FL_WHITE);
	draw_text(db,wx-12,wy-db->textHeight,FL_WHITE,"Phase");
	draw_text(db,wx+wb-40,wy+wh+db->textHeight,FL_WHITE,"Epoche");

	for (i=0; i<stepPhase; i++) 
	{
		double j = (double)i/(double)stepPhase;
		for (x=0; x<wb/5+1; x++)
			draw_line(db,wx+x*5,wy+(int)(wh*j),wx+x*5+1,wy+(int)(wh*j),FL_WHITE);
		draw_line(db,wx-2,wy+(int)(wh*j),wx+2,wy+(int)(wh*j),FL_WHITE);
		sprintf(str,"%4.1f",j-0.5);
		draw_text(db,5,wy+(int)(wh*j),FL_WHITE,str);
	}

	hdif = (double)wh;

	epocheNullJD      = stern.max_jd;
	if (stern.gruppe==STG_BED)
		epocheNullJD += stern.periode/2;
	extremEpocheStart = (int)((extrem_jd0 - epocheNullJD) / stern.periode);
	extremEpocheEnd   = (int)((extrem_jd1 - epocheNullJD) / stern.periode);
	epocheTotalN      = extremEpocheEnd - extremEpocheStart;
	epocheNullX = -(double)extremEpocheStart/(double)epocheTotalN *wb ;
	extremEpocheStep  = (double)wb/(extremEpocheEnd-extremEpocheStart);
	epocheGridStep = 1;
	while (epocheGridStep * 30 < extremEpocheEnd-extremEpocheStart)
		epocheGridStep *= 10;
	if (epocheGridStep * 10 < extremEpocheEnd-extremEpocheStart)
		epocheGridStep *= 3;
	for (i=extremEpocheStart/epocheGridStep; i<=extremEpocheEnd/epocheGridStep; i++)
	{
		x = wx + epocheNullX + i * epocheGridStep * extremEpocheStep;
		draw_line(db, x, wy+wh-2, x, wy+wh+2, FL_WHITE);
		for (y=0; y<wh/5; y++)
			draw_line(db, x,wy+y*5+1, x,wy+y*5, FL_WHITE);
		sprintf(str,"%i",i*epocheGridStep);
		draw_text(db,x-4-4*strlen(str),wy+wh+3,FL_WHITE,str);
	}

	extremPtr = extremFirst;
	while (extremPtr!=NULL) 
	{
		if (extrem_jd0<=extremPtr->datum && extremPtr->datum<=extrem_jd1 && extrem_typ==extremPtr->typ) 
		{
			Pixel color = FL_WHITE; 
			int form = 0;
			double paintEpoche = (extremPtr->datum-epocheNullJD)/stern.periode;
			double paintBR = (extremPtr->datum-epocheNullJD)/stern.periode;
			paintBR -= floor(paintBR);
			if (paintBR<-0.5)	paintBR += 1.0;
			if (paintBR>+0.5)	paintBR -= 1.0;
			paintBR += 0.5;	// map to 0 ... 1
			x = wx + (int)(epocheNullX + paintEpoche * extremEpocheStep);
			y = wy + (int)(hdif*paintBR);
			switch (form) {
			case 0:
				draw_circle_fill(db,x,y,5,color);
				break;
			case 1:
				draw_quad_fill  (db,x,y,5,color);
				break;
			}
		}
		extremPtr = extremPtr->next;
	}

	if (theMode>0)
	{
#ifndef NO_FORMS
		if (fd_graf_phase)
			CURSOR_WATCH(fd_graf_phase->graf_phase->window);
#endif
		switch (theMode) { 
		case 1: {	/* Print */
			FILE *fLst;
			fLst = OpenPrinter();
			PrintScreen(fLst,db->pic->data,grafikDruckX,grafikDruckY,grafikDruckEinschubX,grafikDruckEinschubY);
			ClosePrinter(fLst);
			break; }	
		case 2:		/* Save */
			draw_save(db);
			break;
		case 3:		/* Export */
			break;
		}
#ifndef NO_FORMS
		if (fd_graf_phase)
			CURSOR_NORM(fd_graf_phase->graf_phase->window);
#endif
	}
	draw_free(db);
}


void ExtremGraph_Reset()
{
	struct RExtrem *extremPtr;

	if (extremFirst!=NULL) 
	{
		extrem_jd0 = extremFirst->datum;
		extrem_jd1 = extremFirst->datum;
		extremPtr = extremFirst;
		while (extremPtr->next!=NULL)
		{
			testmin( extremPtr->datum, extrem_jd0 );
			testmax( extremPtr->datum, extrem_jd1 );
			extremPtr = extremPtr->next;
		}
	}
	else
	{
		extrem_jd0 = stern.max_jd+stern.periode;
		extrem_jd1 = stern.max_jd+stern.periode*9;
 	}
}


void ExtremGraph_DateRange( double jd0, double jd1 )
{
	extrem_jd0 = jd0;
	extrem_jd1 = jd1;
}


#ifndef NO_FORMS

int ExtremGraph_Posthandler (FL_OBJECT *ob, int event, FL_Coord mx, FL_Coord my, int key, void *xev)
{
	static char s[64];

	switch (event) {
	case FL_DRAW:
		ExtremGraphPaint(0,NULL);
		break;
	case FL_MOTION:
	case FL_MOUSE:
		sprintf(s,"Epoche: %i", (int)((double)(mx-2-GRAF_HELL_LEFT)*extremEpocheStep+extremEpocheStart) );
		//fl_set_object_label( fd_graf_phase->text, s );
		break;
	}
	return 0;	
}

void ExtremGraphShow()
{
	char str[80];
	
	if ((stern.extremZahl==0) || (stern.periode<=0)) return;
	ExtremGraph_Reset();

	if ((fd_graf_phase!=NULL) && (fd_graf_phase->graf_phase->visible)) 
	{
		fl_hide_form(fd_graf_phase->graf_phase);
	} 
	else 
	{
		fd_graf_phase = create_form_graf_phase();	
		fl_set_form_minsize(fd_graf_phase->graf_phase,200,200);
		fl_set_object_posthandler( fd_graf_phase->grafik, ExtremGraph_Posthandler );
		fl_set_object_label(fd_graf_phase->Close, LS_Close );
	}
	sprintf(str,LS(162),stern.name);
	if (fd_graf_phase_wp.x>0) 
	{
		fl_set_form_position(fd_graf_phase->graf_phase, fd_graf_phase_wp.x,fd_graf_phase_wp.y);
		fl_set_form_size    (fd_graf_phase->graf_phase, fd_graf_phase_wp.w,fd_graf_phase_wp.h);
	}
	fl_show_form(fd_graf_phase->graf_phase,
                ((fd_graf_phase_wp.x>0) ? FL_PLACE_POSITION : FL_PLACE_MOUSE | FL_FREE_SIZE),
                FL_FULLBORDER,str);
	fl_raise_form(fd_graf_phase->graf_phase);
}

#endif


void ExtremConvertBAV( const char* theNameBAV, bool isTypeLong )
{
	FILE *fIn, *fDat;
	char line[256], starName[20], obsCode[5], filename[StrLenPfad];
	double extDate, extMag;
	int  i, extType, extMagI, n=0;
	int  posObserver = (isTypeLong?23:38);

	fIn = fopen( theNameBAV, "r" );
	if (fIn==NULL)
	{
		fprintf(stderr,"Can't load extrema of %s period stars from '%s'\n", (isTypeLong?"long":"short"), theNameBAV );
		return;
	}

	// Format:
	// long:  R    ANDX33347    6.15 PL   0    4  1   14
	//        NAME....TJJJJJJJ MMMMM OBS
	// short: RT   AND50859.4694050859.467003       SHB    25         113 1  121
	//        NAME....JJJJJJJJJJJjjjjjjjjjjjS       OBS                        T
	//                geozent.   heliozent. Stellen   Typ:E(1),RR(2),Cep(3)

	while (fgetline(line,sizeof(line),fIn))
	{
		// scan line of extremum
		starName[0] = tolower(line[5]);
		starName[1] = tolower(line[6]);
		starName[2] = tolower(line[7]);
		for (i=0; i<5; i++)
		{
			if (line[i]==' ')
				starName[i+3] = '\0';
			else
				starName[i+3] = tolower(line[i]);
		}
		starName[8] = '\0';
		for (i=0; i<3; i++)
			obsCode[i] = line[posObserver+i];
		obsCode[3] = '\0';

		if (isTypeLong)
		{
			extMag = atof(line+(line[17]==' '?18:17));
			line[18] = '\0';
			extDate = atof(line+9);
		}
		else
		{
			extMag = 0;
			line[30] = '\0';
			extDate = atof(line+19);
		}
		extDate += 2400000;

		extType = 0;	// 0=Min, 1=Max
		switch (line[(isTypeLong?8:65)])
		{
			case 'X':	extType = 1; break;
			case 'N':	extType = 0; break;
			case '0':	extType = 0; break;
			case '1':	extType = 1; break;
			case '2':	extType = 1; break;
		}

		// append to STE file
		fDat = fopen(ExtremGetFileName(starName,filename),"a");
		//fprintf(stderr,"APPEND %s: ",ExtremGetFileName(starName,filename));
		//fDat = stderr;
		if (fDat==NULL)
		{
			fprintf(stderr,"Warning: Can't append data to file '%s'\n",filename);
		}
		else
		{
			extMagI = (int)(extMag*100);
			fprintf(fDat,"%i\t%f\t%i\t%s\t%s\n",
				extType, extDate, extMagI, obsCode, "" );
			fclose(fDat);
			n++;
		}
	}

	fclose(fIn);
	fprintf(stderr,"Imported %i extrema\n",n);
}


void ExtremReadBAVElements( const char* theFileName )
{
	FILE   *fIn;
	char   line[256], starName[20];
	double element_periode,element_datum,element_max,element_min;
	int    i, isE,isRR;
	struct RDisk *diskStar;

	fIn = fopen( theFileName, "r" );
	if (fIn==NULL)
	{
		fprintf(stderr,"Can't load elements from '%s'\n", theFileName );
		return;
	}

	while (fgetline(line,sizeof(line),fIn))
	{
		// Zeile scannen
		starName[0] = tolower(line[5]);
		starName[1] = tolower(line[6]);
		starName[2] = tolower(line[7]);
		for (i=0; i<5; i++)
		{
			if (line[i]==' ')
				starName[i+3] = '\0';
			else
				starName[i+3] = tolower(line[i]);
		}
		starName[8] = '\0';
		sscanf(line+28,"%lf",&element_periode);
		sscanf(line+13,"%lf",&element_datum);
		element_datum += 2400000;
		sscanf(line+92,"%lf",&element_max);
		sscanf(line+97,"%lf",&element_min);
		isE  = (line[71]=='E');
		isRR = (line[71]=='R' && line[72]=='R');

		// passenden Stern laden
		diskStar = Stern_Laden(starName);
		if (diskStar==NULL)
		{
			fprintf(stderr,"Stern %s nicht gefunden!\n",starName);
			if (ExtremExists(starName))
			{	// Stern neu anlegen...
				char stb[4];
				strn0cpy(stb,starName,3);
				fprintf(stderr,"Erzeuge Stern %s\n",starName);
				Daten_Stern_Neu( stb, starName+3 );
				Daten_GCVS_Suche( stb, starName+3 );
				diskStar = diskLoad;
			}
		}
		if (diskStar)
		{
			stern.periode = element_periode;
			stern.max_jd  = element_datum;
			if (isE)	stern.max_jd += element_periode/2;
			Daten_Speichern();
			fprintf(stderr,"Elemente gelesen fuer Stern %s\n",starName);
		}
	}

	fclose(fIn);
}


